# chat window class
# coding: utf-8

##   Copyright (C) 2010 Coopi <gnu@coopi.org>
## 
##   This program is free software; you can redistribute it and/or
##   modify it under the terms of the GNU General Public License as
##   published by the Free Software Foundation; either version 2 of the
##   License, or (at your option) any later version.
## 
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##   General Public License for more details.
## 
##   You should have received a copy of the GNU General Public
##   License along with this program; if not, write to the
##   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import pango

import os
import datetime

class chat:

	# save the conversation
	def save_conversation(self):
		body = self.pygeon.remove_emoticons_from_message(self.txtBody)

		if len(body) == 0: return # exit if the chat is empty

		# save the conversation into db if needed
		if self.pygeon.conversations_save == 0 or (self.pygeon.conversations_save == 2 and self.pygeon.message_box(_("Do you want to save this conversation?"), _("Saving conversation"), gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, self.winChat) == gtk.RESPONSE_YES):
			cursor = self.pygeon.conn.cursor()
			cursor.execute("insert into chats (date, with, with_name, body) values (?, ?, ?, ?)", (datetime.datetime.now().strftime("%d-%m-%Y %H.%M.%S"), self.pygeon.tdb(self.to), self.pygeon.tdb(self.name), self.pygeon.tdb(body.decode('utf-8'))))
			self.pygeon.conn.commit()
			cursor.close()

	# window closing
	def delete_event(self, widget, event):
		self.save_conversation()
		self.pygeon.remove_chat(self.thread)
		if len(self.pygeon.read_message_classes) == 0 and len(self.pygeon.send_message_classes) == 0 and len(self.pygeon.crono_classes) == 0 and len(self.pygeon.chats) == 0: self.pygeon.unload_emoticons() # unload emoticons if necessary
		self.winChat.destroy()

	# click on close button
	def cmdClose_clicked(self, widget):
		self.save_conversation()
		self.pygeon.remove_chat(self.thread)
		if len(self.pygeon.read_message_classes) == 0 and len(self.pygeon.send_message_classes) == 0 and len(self.pygeon.crono_classes) == 0 and len(self.pygeon.chats) == 0: self.pygeon.unload_emoticons() # unload emoticons if necessary
		self.winChat.destroy()

	# click on send button
	def cmdSend_clicked(self, widget):
		import xmpp

		# don't send if the user isn't connected
		if self.pygeon.cboStatus.get_active() == 3 or not self.pygeon.client.isConnected():
			self.pygeon.message_box(_("You must be connected to send the message"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winChat)
			return

		# remove emoticons from message
		plain_message = self.pygeon.remove_emoticons_from_message(self.txtMessage).strip('\n ')
		length = len(plain_message)

		# go ahead if the message isn't empty and isn't too long
		if length > 0 and length <= self.pygeon.max_message_length:
			# add timestamp to textview
			self.pygeon.add_timestamp_to_chat(self.txtBody, self.pygeon.myname, "timestamp_own")

			# get the current length of textview
			offset = self.txtBody.get_buffer().get_char_count()

			# add plain message into the textview
			self.txtBody.get_buffer().insert(self.txtBody.get_buffer().get_end_iter(), plain_message)

			# url mapping
			self.pygeon.add_url_to_message(self.txtBody, offset)

			# add emoticons to message
			self.pygeon.add_emoticons_to_message(self.txtBody, offset)

			# send the message to the server
			message = xmpp.protocol.Message(to = self.to, body = plain_message, typ = "chat", frm = self.pygeon.jid)
			message.setThread(self.thread)
			self.pygeon.client.send(message)

			# scroll to end
			self.txtBody.scroll_to_mark(self.txtBody.get_buffer().create_mark(None, self.txtBody.get_buffer().get_end_iter(), False), 0.0)

			# clear the textbox and grab focus
			self.txtMessage.get_buffer().set_text("")
			self.txtMessage.grab_focus()
		elif length > self.pygeon.max_message_length: # warn if the message is too long
			self.pygeon.message_box(_("The message exceeded the maximum lenght of ") + str(self.pygeon.max_message_length) + _(" characters"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winChat)
			self.txtMessage.grab_focus()
	
	# click on the emoticon
	def emoticon_clicked(self, widget, path):
		if self.pygeon.debug: print _("You clicked on the emoticon: ") + path + _(" which corresponds to: ") + self.pygeon.emoticons_symbol[self.pygeon.emoticons_path.index(path)]

		# add the pixbuf to the textview. if it doesn't exist it will be created
		if self.pygeon.emoticons_pixbuf[self.pygeon.emoticons_path.index(path)] is None:
			self.pygeon.emoticons_pixbuf[self.pygeon.emoticons_path.index(path)] = gtk.gdk.pixbuf_new_from_file(path)
			if self.pygeon.debug: print _("I created the pixbuf for: ") + self.pygeon.emoticons_symbol[self.pygeon.emoticons_path.index(path)]
		
		iter = self.txtMessage.get_buffer().get_iter_at_offset(self.txtMessage.get_buffer().get_property("cursor_position"))
		self.txtMessage.get_buffer().insert_pixbuf(iter, self.pygeon.emoticons_pixbuf[self.pygeon.emoticons_path.index(path)])

		# destroy the window
		self.winEmoticon.destroy()

		# grab focus on textview
		self.txtMessage.grab_focus()

	# emoticon window closing
	def winEmoticon_delete(self, widget, event):
		self.winEmoticon.destroy()
		
		# grab focus on textview
		self.txtMessage.grab_focus()
		
	# open emoticon window
	def cmdEmoticon_clicked(self, widget):
		self.winEmoticon = gtk.Window()
		vbox = gtk.VBox()

		hbox1 = gtk.HBox()
		hbox2 = gtk.HBox()
		hbox3 = gtk.HBox()
		hbox4 = gtk.HBox()
		hbox5 = gtk.HBox()
		hbox6 = gtk.HBox()

		vbox.pack_start(hbox1)
		vbox.pack_start(hbox2)
		vbox.pack_start(hbox3)
		vbox.pack_start(hbox4)
		vbox.pack_start(hbox5)
		vbox.pack_start(hbox6)

		# load emoticons
		for file in self.pygeon.emoticons_path:
			image = gtk.Image()
			image.set_from_file(file)
			button = gtk.Button()
			button.set_image(image)

			if len(hbox1.get_children()) < 5: hbox1.pack_start(button, False)
			elif len(hbox2.get_children()) < 5: hbox2.pack_start(button, False)
			elif len(hbox3.get_children()) < 5: hbox3.pack_start(button, False)
			elif len(hbox4.get_children()) < 5: hbox4.pack_start(button, False)
			elif len(hbox5.get_children()) < 5: hbox5.pack_start(button, False)
			else: hbox6.pack_start(button, False)

			# button signal
			button.connect("clicked", self.emoticon_clicked, file)
			
		self.winEmoticon.add(vbox)
		self.winEmoticon.set_modal(True)
		self.winEmoticon.set_icon_from_file(os.path.normpath(self.pygeon.icon_application_path + "pygeon-small.png"))
		self.winEmoticon.set_resizable(False)
		self.winEmoticon.set_title(":)")

		self.winEmoticon.connect("delete_event", self.winEmoticon_delete)
		self.winEmoticon.show_all()

	# load avatar if any
	def load_avatar(self):
		if os.path.isfile(os.path.normpath(self.pygeon.pref_path + "/avatars/" + self.to + "_small")):
			self.imgAvatar.set_from_file(os.path.normpath(self.pygeon.pref_path + "/avatars/" + self.to + "_small"))
		else: # if there is no avatar hide the image
			self.imgAvatar.hide()

	# enter key in textview
	def txtMessage_key_press(self, widget, event):
		if gtk.gdk.keyval_name(event.keyval) == "Return" and event.state != gtk.gdk.CONTROL_MASK:
			self.cmdSend.clicked()
			return True

	# when the window receive focus
	def winChat_focus_in(self, widget, event):
		# remove urgency_hint if present
		if self.winChat.get_urgency_hint(): self.winChat.set_urgency_hint(False)

	# click on url
	def txtBody_clicked(self, widget, event):
		self.pygeon.url_clicked(self.txtBody, event)

	# mouse move into textbox
	def txtBody_mouse_move(self, widget, event):
		self.pygeon.url_move_on(self.txtBody, event)

	# constructor
	def __init__(self, pygeon, to, thread, body=None):
		self.pygeon = pygeon # main window
		self.to = to # username
		self.name = self.pygeon.get_name_from_username(to)
		self.thread = thread

		self.gladefile = "../data/glade/pygeon.glade"
		self.winContainer = gtk.glade.XML(self.gladefile, "winChat")

		# glade objects
		self.winChat = self.winContainer.get_widget("winChat")
		self.cmdClose = self.winContainer.get_widget("cmdChatClose")
		self.cmdSend = self.winContainer.get_widget("cmdChatSend")
		self.cmdEmoticon = self.winContainer.get_widget("cmdChatEmoticon")
		self.imgAvatar = self.winContainer.get_widget("imgChatAvatar")
		self.lblTo = self.winContainer.get_widget("lblChatTo")
		self.lblChatSend = self.winContainer.get_widget("lblChatSend")
		self.lblChatClose = self.winContainer.get_widget("lblChatClose")
		self.imgEmoticon = self.winContainer.get_widget("imgChatEmoticon")
		self.txtMessage = self.winContainer.get_widget("txtChatMessage")
		self.txtBody = self.winContainer.get_widget("txtChatBody")
		self.imgSend = self.winContainer.get_widget("imgChatSend")
		self.imgClose = self.winContainer.get_widget("imgChatClose")

		# hide window if chat has been received and the present option isn't enabled
		if not body is None and not self.pygeon.conversations_present:
			self.winChat.hide()

		# glade objects
		self.lblChatSend.set_text(_("Send"))
		self.lblChatClose.set_text(_("Close"))

		# signals
		self.winChat.connect("delete_event", self.delete_event)
		self.winChat.connect("focus_in_event", self.winChat_focus_in)
		self.cmdClose.connect("clicked", self.cmdClose_clicked)
		self.cmdSend.connect("clicked", self.cmdSend_clicked)
		self.cmdEmoticon.connect("clicked", self.cmdEmoticon_clicked)
		self.txtMessage.connect("key_press_event", self.txtMessage_key_press)
		self.txtBody.connect("button_press_event", self.txtBody_clicked)
		self.txtBody.connect("motion_notify_event", self.txtBody_mouse_move)

		self.imgEmoticon.set_from_file(os.path.normpath(self.pygeon.icon_theme_emoticons_path + "face-smile.png"))
		self.imgSend.set_from_file(os.path.normpath(self.pygeon.icon_theme_application_path + "message_big.png"))
		self.imgClose.set_from_file(os.path.normpath(self.pygeon.icon_theme_application_path + "close.png"))

		self.winChat.set_title(_("Chat with ") + self.name)
		self.winChat.set_icon_from_file(os.path.normpath(self.pygeon.icon_application_path + "pygeon-small.png"))

		# create textview tag
		self.txtBody.get_buffer().create_tag("timestamp_own", foreground = "#aa0000", size_points = 8)
		self.txtBody.get_buffer().create_tag("timestamp_other", foreground = "#0000aa", size_points = 8)
		self.txtBody.get_buffer().create_tag("status", foreground = "#888888", size_points = 8, justification = gtk.JUSTIFY_CENTER)
		self.txtBody.get_buffer().create_tag("url", foreground = "blue", underline = pango.UNDERLINE_SINGLE)

		self.lblTo.set_markup("<b>" + self.name + "</b>")
		self.load_avatar()

		# load emoticons lists if needed
		if len(self.pygeon.read_message_classes) == 0 and len(self.pygeon.send_message_classes) == 0 and len(self.pygeon.crono_classes) == 0 and len(self.pygeon.chats) == 0: self.pygeon.load_emoticons()

		# if the body is specified append it to the textview
		if not body is None:
			# add timestamp to the textview
			self.pygeon.add_timestamp_to_chat(self.txtBody, self.name, "timestamp_other")

			# get textview length
			offset = self.txtBody.get_buffer().get_char_count()

			# add message into the textview
			self.txtBody.get_buffer().insert(self.txtBody.get_buffer().get_end_iter(), body)
			
			# url mapping
			self.pygeon.add_url_to_message(self.txtBody, offset)

			# add emoticons to message
			self.pygeon.add_emoticons_to_message(self.txtBody, offset)

		self.txtMessage.grab_focus()
