#! /usr/bin/env python
# coding: utf-8
# main class of pygeon

##   Copyright (C) 2010 Coopi <gnu@coopi.org>
## 
##   This program is free software; you can redistribute it and/or
##   modify it under the terms of the GNU General Public License as
##   published by the Free Software Foundation; either version 2 of the
##   License, or (at your option) any later version.
## 
##   This program is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##   General Public License for more details.
## 
##   You should have received a copy of the GNU General Public
##   License along with this program; if not, write to the
##   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import pango
import gobject
import os
import sys
import xmpp
import base64
import notify
import socket

import signal
import time
import datetime
import string

try:
	import winsound
except Exception:
	pass

from pysqlite2 import dbapi2 as sqlite

# localization
import gettext, locale

import __builtin__
__builtin__._ = gettext.gettext

# array of supported languages (dictionary)
languages = {'Italiano': 'it', 'English': 'en', 'Deutsch': 'de', 'Türkçe': 'tr', 'Español': 'es', 'Русский': 'ru', 'Français': 'fr', 'Ελληνικά': 'el' }
languages_reverse = { 'it': 'Italiano', 'en': 'English', 'de': 'Deutsch', 'tr': 'Türkçe', 'es': 'Español', 'ru': 'Русский', 'fr': 'Français', 'el': 'Ελληνικά' }

# load the system language (only on windows because on linux is already loaded by default)
if os.name == 'nt':
    lang = os.getenv('LANG')
    
    if lang is None:
        lang, default_enc = locale.getdefaultlocale()
        
    if lang:
        os.environ['LANG'] = lang

# get the language alias
if not os.environ['LANG'] is None:
	os.environ['LANG'] = os.environ['LANG'].split('.')[0].split('_')[0]

# load the language into preferences if it exists, otherwise it use the system language
lang = None
conn = None
if os.name == 'posix' and os.path.isfile(os.path.expanduser('~') + "/.pygeon/pygeon.db"):
	conn = sqlite.connect(os.path.expanduser('~') + "/.pygeon/pygeon.db")
elif not os.name == 'posix' and os.path.isfile(os.environ['APPDATA'] + "\\pygeon\\pygeon.db"):
	conn = sqlite.connect(os.environ['APPDATA'] + "\\pygeon\\pygeon.db")

if not conn is None:
	cursor = conn.cursor()
	cursor.execute("select language from preferences where id = 1")
	lang = cursor.fetchone()[0].encode('utf-8')
	cursor.close()
	conn.close()

if not lang is None:
	try:	
		os.environ['LANG'] = languages[lang]
	except:
		pass

# if the used language (of preferences or system) doesn't exist load english
if not os.environ['LANG'] in languages_reverse:
	os.environ['LANG'] = 'en'

try:
	my_locale = gettext.translation('pygeon', os.getcwd() + "/./../data/locale")
except:
	# if the program doesn't found the localization in the current folder, search it in the system folder (only for linux)
	my_locale = gettext.translation('pygeon', "/usr/share/locale")

my_locale.install()
# end localization

# import the other classes
from info import info
from pref import pref
from tray import tray
from chat import chat
from crono import crono
from send_message import send_message
from read_message import read_message
from user_info import user_info
from file_transfer import file_transfer
from file_manager import file_manager

##################################################################
########################### setup ################################



##################################################################

class pygeon:

	# constant
	max_message_length = 3000 # maximum length of a message (both normal and chat)
	program_version = "1.1.1" # version number

	# paths
	pref_path = "" # preferences path
	current_path = "" # current program path
	icon_application_path = "" # generic icon application path
	icon_theme_application_path = "" # theme icon application path
	icon_theme_status_path = "" # theme icon status path
	icon_theme_emoticons_path = "" # theme emoticons path

	debug = False # simple debug
	ddebug = False # full debug
	mute = False # mute mode
	pref_class = None # preferences class when the preferences window is opened
	jid = None # jabber id
	client = None # jabber client
	message_counter = False # semaphore for counting of messages
	presence_message = "" # presence message

	# emoticons variable
	emoticons_symbol = []
	emoticons_path = []
	emoticons_pixbuf = []

	# class lists
	chats = [] # chat list
	send_message_classes = [] # send message list
	read_message_classes = [] # read message list
	crono_classes = [] # crono list
	file_transfers = [] # file transfer class (both send and receive)
	file_manager_class = None # file manager class (only one)
	
	check_unread = False # semaphore for unread messages
	unread_messages_count = 0 # number of unread messages
	network_down = False # control variable
	reconnecting = False # control variable
	just_connected = True # control variable
	is_connecting = False # semaphore to avoid multiple connections to the server
	previous_check = None # unixtime of the check_network function

	# options with default value (they will be load by load_preferences function)
	# account
	myname = ""
	username = ""
	password = ""
	server = ""
	domain = ""
	port = 5222
	autoconnect = False
	avatar = ""
	avatar_temp = "" # temp path waiting the ok by the server to receive the binval
	avatar_uploaded = False # control variable
	sort_method = 1 # sort method

	# files
	files_path = ""
	files_accept = 2 # 0=accept always, 1=refuse always, 2=ask for confirmation

	# sounds
	sounds_receive_path = ""
	sounds_receive = True
	sounds_connect_path = ""
	sounds_connect = False
	sounds_disconnect_path = ""
	sounds_disconnect = False

	# notifications
	notify_normal = True
	notify_chat = True
	notify_connect = False
	notify_disconnect = False
	notify_status = False
	notify_add = True
	notify_remove = True
	notify_transfer = True

	# conversations
	conversations_save = 0 # 0=save always, 1=don't save, 2=ask for confirmation
	conversations_present = False

	# language and theme
	program_language = 'English'
	theme = 'default'

	# database
	conn = None

	# notification instance
	notifier = notify.NotificationStack()
	notifier.edge_offset_x = 10
	notifier.edge_offset_y = 30
	notifier.bg_color = gtk.gdk.Color(red=62965, green=54741, blue=42405)

	# connect to db
	def connect_to_db(self):
		return sqlite.connect(self.pref_path + "/pygeon.db")

	# close db connection
	def disconnect_from_db(self, conn):
		conn.close()

	# remove the zombie process 'p' when the maximum time 'max_time' is reached or when the execution finished
	def kill_zombies(self, p, max_time):
		while p.poll() < 0:
			while gtk.events_pending(): gtk.main_iteration()
      
			if max_time > 0.5:
				max_time -= 0.25
				time.sleep(0.25)
			else:  # still there, force kill
				os.kill(p.pid, signal.SIGKILL)
				time.sleep(0.5)
				p.poll() # final try
				break

	# get the desktop environment in use (gnome, kde, xfce)
	# code by gajim
	def get_desktop_environment(self):
		# get processes
		if os.path.isdir('/proc'):
			# get content of /proc
			files = os.listdir('/proc')

			# filter
			files = filter(str.isdigit, files)
			files = filter(lambda f:os.path.isdir('/proc/' + f), files)
			files = filter(lambda f:os.access('/proc/' + f +'/exe', os.F_OK), files)
			files = filter(lambda f:os.path.islink('/proc/' + f + '/exe'), files)

			# get processes after the filter
			processes = [os.path.basename(os.readlink('/proc/' + f +'/exe')) for f in files]

			if "gnome-session" in processes: return "gnome" # gnome2
			elif "kdeinit" in processes or "kdeinit4" in processes: return "kde" # kde3 e kde4
			elif "startxfce4" in processes or "xfce4-session" in processes: return "xfce" # xfce4
			else: return ""

		return ""

	# click on url
	def url_clicked(self, txtBody, event):
		# get buffer coordinates
		coords = txtBody.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, int(event.x), int(event.y))

		# get iter tags
		iter = txtBody.get_iter_at_location(coords[0], coords[1])
		tags = iter.get_tags()

		# if the tag url is found open the link
		for tag in tags:
			if tag.get_property('name') == "url":
				url = ""

				# move to tag start and read all the url
				if not iter.begins_tag(tag):
					while iter.backward_char():
						if iter.begins_tag(tag):
							break
						
				url = iter.get_char()

				while iter.forward_char():
					if iter.ends_tag(tag):
						break

					url = url + iter.get_char()

				self.open_url(url)

				if self.debug: print _("The url you clicked on is: ") + url

				break

	# when the mouse is on an url change cursor
	def url_move_on(self, txtBody, event):
		# get buffer coordinates
		coords = txtBody.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, int(event.x), int(event.y))

		# get iter tags
		iter = txtBody.get_iter_at_location(coords[0], coords[1])
		tags = iter.get_tags()

		# search for url tag
		found = False
		for tag in tags:
			if tag.get_property('name') == "url":
				txtBody.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
				found = True
				break

		if not found: txtBody.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))

	# encode strings that are read from db
	def fdb(self, s):
		return s.encode('utf-8')

	# decode strings that are going to be put into db
	def tdb(self, s):
		return unicode(s)

	# constructor
	def __init__(self, pygeon, to, thread, body=None):
		self.pygeon = pygeon # main class
		self.to = to # username
		self.name = self.pygeon.get_name_from_username(to)
		self.thread = thread

		self.gladefile = "../data/glade/pygeon.glade"

	# load emoticons variables (symbol and path)
	def load_emoticons(self):
		for emoticon in os.listdir(self.icon_theme_emoticons_path):
			if emoticon == "face-angel.png": self.emoticons_symbol.append("O)")
			elif emoticon == "face-crying.png": self.emoticons_symbol.append("P(")
			elif emoticon == "face-devilish.png": self.emoticons_symbol.append("#D")
			elif emoticon == "face-glasses.png": self.emoticons_symbol.append("B)")
			elif emoticon == "face-grin.png": self.emoticons_symbol.append(":D")
			elif emoticon == "face-kiss.png": self.emoticons_symbol.append(":X")
			elif emoticon == "face-monkey.png": self.emoticons_symbol.append(":8")
			elif emoticon == "face-plain.png": self.emoticons_symbol.append(":|")
			elif emoticon == "face-sad.png": self.emoticons_symbol.append(":(")
			elif emoticon == "face-smile.png": self.emoticons_symbol.append(":)")
			elif emoticon == "face-smile-big.png": self.emoticons_symbol.append(":>")
			elif emoticon == "face-surprise.png": self.emoticons_symbol.append(":O")
			elif emoticon == "face-wink.png": self.emoticons_symbol.append(";)")
			elif emoticon == "face-raspberry.png": self.emoticons_symbol.append(":P")
			elif emoticon == "bear.png": self.emoticons_symbol.append("#B")
			elif emoticon == "flower.png": self.emoticons_symbol.append("#F")
			elif emoticon == "heidy.png": self.emoticons_symbol.append("#H")
			elif emoticon == "face-cool.png": self.emoticons_symbol.append("#8")
			elif emoticon == "face-angry.png": self.emoticons_symbol.append("#(")
			elif emoticon == "face-clown.png": self.emoticons_symbol.append("#0")
			elif emoticon == "face-confused.png": self.emoticons_symbol.append(":?")
			elif emoticon == "face-embarrassed.png": self.emoticons_symbol.append(":@")
			elif emoticon == "face-ninja.png": self.emoticons_symbol.append("#I")
			elif emoticon == "face-pir.png": self.emoticons_symbol.append("#;")
			elif emoticon == "face-quiet.png": self.emoticons_symbol.append(":!")
			elif emoticon == "face-sick.png": self.emoticons_symbol.append(":]")
			elif emoticon == "face-sleeping.png": self.emoticons_symbol.append("#Z")
			elif emoticon == "face-uncertain.png": self.emoticons_symbol.append(":~")
			elif emoticon == "face-worried.png": self.emoticons_symbol.append("#[")
			elif emoticon == "face-yawn.png": self.emoticons_symbol.append("#O")
			
			self.emoticons_path.append(os.path.normpath(self.icon_theme_emoticons_path + emoticon))
			self.emoticons_pixbuf.append(None) # create the pixbuf list without content to save memory

		if self.debug:
			n = 0
			while n < len(self.emoticons_symbol):
				print self.emoticons_symbol[n] + " => " + self.emoticons_path[n]
				n = n + 1

	# empty emoticons list (symbol, path and pixbuf)
	def unload_emoticons(self):
		self.emoticons_symbol = []
		self.emoticons_path = []
		self.emoticons_pixbuf = []

		if self.debug:
			print _("Elements in symbol: ") + str(len(self.emoticons_symbol))
			print _("Elements in path: ") + str(len(self.emoticons_path))
			print _("Elements in pixbuf: ") + str(len(self.emoticons_pixbuf))
			print _("The emoticons lists have been flushed")

	# create the progress bar
	def create_progress(self, progress, steps, text):
		if steps > 0: progress.set_property("discrete_blocks", steps)
		if text: progress.set_text("0 %")
		progress.set_fraction(0)

	# destroy the progress bar
	def destroy_progress(self, progress, text):
		if text: progress.set_text(_("Idle"))
		progress.set_fraction(0)

	# set step for progress bar
	def set_progress(self, progress, step, text):
		try:
			if step <= progress.get_property("discrete_blocks") and step >= 0:
				progress.set_fraction(step / (progress.get_property("discrete_blocks") - 1.0))
				if text: progress.set_text(str(int(progress.get_fraction() * 100)) + " %")
		except Exception:
			self.destroy_progress(progress, text)
	
	# set step for file transfer progress bar
	def set_progress_transfer(self, progress, step, bytes, tot, text):
		try:
			if step <= progress.get_property("discrete_blocks") and step >= 0:
				progress.set_fraction(step / (progress.get_property("discrete_blocks") - 1.0))

				# calculate total and current size
				total = self.get_human_size(tot)
				current = self.get_human_size(bytes)

				progress.set_text(current + _(" of ") + total + " " + str(int(progress.get_fraction() * 100)) + " %")
		except Exception:
			self.destroy_progress(progress, text)

	# print size in human readable mode
	def get_human_size(self, s):
		if s < 1048576:
			return "%.2f" % (s / 1024.0) + _(" Kb")
		elif s >= 1048576 and s < 1073741824:
			return "%.2f" % (s / 1048576.0) + _(" Mb")
		else:
			return "%.2f" % (s / 1073741824.0) + _(" Gb")

	# save sort method to db
	def save_sort_method(self):
		cursor = self.conn.cursor()
		cursor.execute("update preferences set sort_method = ? where id = 1", (int(self.sort_method),))
		self.conn.commit()
		cursor.close()

	# close all chat windows
	def close_all_chats(self):
		for chat in reversed(self.chats):
			chat.cmdClose.clicked()

	# open url 'url' depending on the system
	def open_url(self, url):
		if url.startswith("www."): url = "http://" + url # add http:// to the start of the string if not present

		try:
			if os.name == 'nt':
				os.startfile(url)
			elif os.name == 'posix':
				desktop = self.get_desktop_environment()

				import subprocess

				if desktop == "gnome": p = subprocess.Popen("gnome-open " + url, shell = True)
				elif desktop == "kde": p = subprocess.Popen("kfmclient exec " + url, shell = True)
				elif desktop == "xfce": p = subprocess.Popen("exo-open " + url, shell = True)

				self.kill_zombies(p, 10) # kill zombie process in a maximum time of 10 seconds
		except Exception:
			if self.debug: print _("An error occured at the opening of ") + url

	# add timestamp to the textview of the chat with user 'utente' and formatted with tag 'tag'
	def add_timestamp_to_chat(self, textview, utente, tag):
		if textview.get_buffer().get_char_count() == 0:
			timestamp = "[" + datetime.datetime.now().strftime("%H:%M:%S") + "] " + utente + ": "
		else:
			timestamp = "\n[" + datetime.datetime.now().strftime("%H:%M:%S") + "] " + utente + ": "

		textview.get_buffer().insert_with_tags_by_name(textview.get_buffer().get_end_iter(), timestamp, tag)

	# map timestamps and message status
	# utente is the fullname of the user, not the username
	def map_timestamps_and_status_to_message(self, utente, message):
		# get message coded into utf8
		messaggio = unicode(message.get_buffer().get_text(message.get_buffer().get_start_iter(), message.get_buffer().get_end_iter()), "utf-8")

		# get all lines of the message
		lines = messaggio.split('\n')

		# for each line check if apply the tag timestamp
		pos = 0
		for line in lines:
			# if it contain [hh:mm:ss] c: (c is the username followed by : and a space)
			if len(line) > 13 and line[0] == "[" and line[3] == ":" and line[6] == ":" and line[9] == "]" and line[10] == " ":
				# search the ": " string after the username
				try:
					char_pos = line.index(": ")
				except Exception:
					char_pos = -1

				# get the user fullname
				name = line[11:char_pos]

				if char_pos != -1: # it's a timestamp, apply the tag
					if name != utente: # if it doesn't combine use the own tag
						message.get_buffer().apply_tag_by_name("timestamp_own", message.get_buffer().get_iter_at_line(pos), message.get_buffer().get_iter_at_line_offset(pos, char_pos))
					else: # otherwise use other tag
						message.get_buffer().apply_tag_by_name("timestamp_other", message.get_buffer().get_iter_at_line(pos), message.get_buffer().get_iter_at_line_offset(pos, char_pos))
			elif line.startswith(utente + _(" is Available")) or line.startswith(utente + _(" is Away")) or line.startswith(utente + _(" is Not available")) or line.startswith(utente + _(" is Offline")): # apply the status tag
				message.get_buffer().apply_tag_by_name("status", message.get_buffer().get_iter_at_line(pos), message.get_buffer().get_iter_at_line_offset(pos, len(line)))

			pos = pos + 1
				
	# apply filter tag to a message when it find 'filter'
	def apply_filter_to_message(self, message, filter):
		# get utf8 message
		messaggio = unicode(message.get_buffer().get_text(message.get_buffer().get_start_iter(), message.get_buffer().get_end_iter()), "utf-8")

		pos = messaggio.find(filter, 0)
		while pos != -1:
			message.get_buffer().apply_tag_by_name("filter", message.get_buffer().get_iter_at_offset(pos), message.get_buffer().get_iter_at_offset(pos + len(filter)))

			pos = messaggio.find(filter, pos + len(filter) + 1)

	# add a message to notify a status change to all opened chat with 'utente'
	def add_status_to_chat(self, utente, status, mex=None):
		name = self.get_name_from_username(utente)

		message = "\n" + name + _(" is ") + status
		if not mex is None: message = message + " (" + mex + ")"

		for chat in self.chats:
			if (chat.to == utente or str(self.jid).split('/')[0] == utente) and chat.txtBody.get_buffer().get_char_count() > 0:
				chat.txtBody.get_buffer().insert_with_tags_by_name(chat.txtBody.get_buffer().get_end_iter(), message, "status")

	# parses message starting from offset and converts the url into links
	def add_url_to_message(self, message, offset=0):
		# get utf8 message
		messaggio = unicode(message.get_buffer().get_text(message.get_buffer().get_iter_at_offset(offset), message.get_buffer().get_end_iter()), "utf-8")

		# not valid characters for url (if it found one of this characters consider the url completed)
		invalid_chars = [" ", ")", "(", "[", "]", "<", ">", "|", ",", "\n"]

		max = len(messaggio)
		i = 0

		while i < max:
			www = http = https = ftp = ""

			# get the first 4, 6, 7, 8 characters if possible
			if i < max - 4: www = messaggio[i] + messaggio[i+1] + messaggio[i+2] + messaggio[i+3]
			if i < max - 6: ftp = messaggio[i] + messaggio[i+1] + messaggio[i+2] + messaggio[i+3] + messaggio[i+4] + messaggio[i+5]
			if i < max - 7: http = messaggio[i] + messaggio[i+1] + messaggio[i+2] + messaggio[i+3] + messaggio[i+4] + messaggio[i+5] + messaggio[i+6]
			if i < max - 8: https = messaggio[i] + messaggio[i+1] + messaggio[i+2] + messaggio[i+3] + messaggio[i+4] + messaggio[i+5] + messaggio[i+6] + messaggio[i+7]

			# check if it's an url
			if www.lower() == "www.": length = 4
			elif ftp.lower() == "ftp://": length = 6
			elif http.lower() == "http://": length = 7
			elif https.lower() == "https://": length = 8
			else: length = 0

			# if an url is found go on
			if length > 0:
				# go on until it find a space, a non-valid character or the message finish
				end = i + length
				while end < max:
					if messaggio[end] in invalid_chars: break # if it's a non-valid character stop
					else: end = end + 1

				url = message.get_buffer().get_text(message.get_buffer().get_iter_at_offset(offset + i), message.get_buffer().get_iter_at_offset(offset + end))

				# apply tag to founded url
				message.get_buffer().apply_tag_by_name("url", message.get_buffer().get_iter_at_offset(offset + i), message.get_buffer().get_iter_at_offset(offset + end))

				i = end + 1 # go to end of the string

				if self.debug: print _("I found the url: ") + url
			else: i = i + 1

	# parse the message and add the emoticons
	def add_emoticons_to_message(self, message, offset=0):
		position_list = [] # emoticons positions
		index_list = [] # index of emoticons in symbol, path and pixbuf lists

		# scroll all the message searching for emoticons
		# code the string to utf8 because the chars aren't bytes
		messaggio = unicode(message.get_buffer().get_text(message.get_buffer().get_iter_at_offset(offset), message.get_buffer().get_end_iter()), "utf-8")

		max = len(messaggio)
		for i in range(max):
			if i < max-1:
				try:
					pos = self.emoticons_symbol.index(messaggio[i] + messaggio[i+1])
				except ValueError:
					pos = -1

				# if it find the emoticon save the position
				if pos != -1:
					position_list.append(offset + i)
					index_list.append(pos)
			else: break

		if self.debug:
			for i in range(len(position_list)):
				print _("Found ") + self.emoticons_symbol[index_list[i]] + _(" in position ") + str(position_list[i])

		# add founded emoticons in the right position
		for i in reversed(range(len(position_list))):
			iter = message.get_buffer().get_iter_at_offset(position_list[i])
		
			# create the pixbuf if it doesn't exist
			if self.emoticons_pixbuf[index_list[i]] is None:
				self.emoticons_pixbuf[index_list[i]] = gtk.gdk.pixbuf_new_from_file(self.emoticons_path[index_list[i]])
				if self.debug: print _("I created the pixbuf for: ") + self.emoticons_symbol[index_list[i]]

			# add the emoticon 
			message.get_buffer().insert_pixbuf(iter, self.emoticons_pixbuf[index_list[i]])
			
			# remove text
			start_iter = message.get_buffer().get_iter_at_offset(position_list[i]+1)
			end_iter = message.get_buffer().get_iter_at_offset(position_list[i]+3)

			message.get_buffer().delete(start_iter, end_iter)

	# remove emoticons from message and insert symbols
	# return the converted message
	def remove_emoticons_from_message(self, message):
		messaggio = ""

		for i in range(message.get_buffer().get_char_count()):
			iter = message.get_buffer().get_iter_at_offset(i)

			if iter.get_pixbuf() is None: messaggio = messaggio + iter.get_char()
			else:
				try:
					messaggio = messaggio + self.emoticons_symbol[self.emoticons_pixbuf.index(iter.get_pixbuf())]
				except ValueError:
					pass

		return messaggio

	# play sound 'sound'
	# if force is True the sound is played without check just_connected
	# return True if success, False otherwise
	def play_sound(self, sound, force=False):
		try:
			if force or not self.just_connected:
				if sound.endswith("wav"):
					if os.name == 'posix':
						import subprocess

						p = subprocess.Popen(["aplay", "-q", sound])
						self.kill_zombies(p, 5) # kill zombie process in a maximum time of 5 seconds
					else: winsound.PlaySound(sound, winsound.SND_FILENAME|winsound.SND_ASYNC)
			
					return True
				else: return False
			else:
				return False
		except Exception:
			pass

	# get local ip address
	def get_local_ip(self):
		s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		s.connect((self.server, self.port))
		ip = s.getsockname()[0]
		s.close()

		return ip

	# window closing
	def delete_event(self, widget, event, data=None):
		# hide the window but doesn't close it (it's in the tray)
		self.winMain.hide()
		return True

	# window destroy
	def destroy(self, widget, data=None):	
		self.close_program()
		gtk.main_quit()

	# quit application
	def mnuEsci_activate(self, widget, data=None):
		self.close_program()
		gtk.main_quit()
	
	# open the send broadcast window
	def mnuBroadcast_activate(self, widget, data=None):
		if self.cboStatus.get_active() != 3 and self.client.isConnected(): # check the user isn't offline
			self.send_message_classes.append(send_message(self))
		else:
			self.message_box(_("You must be connected to send a broadcast"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winMain)

	# open the user chooser window to send the multiple message
	def mnuMulti_activate(self, widget):
		if self.cboStatus.get_active() != 3 and self.client.isConnected(): # check the user isn't offline
			# crea the window
			win = gtk.Window()

			vbox = gtk.VBox()

			scroll = gtk.ScrolledWindow()
			scroll.set_property("hscrollbar_policy", gtk.POLICY_NEVER)
			scroll.set_property("vscrollbar_policy", gtk.POLICY_AUTOMATIC)

			lstUsers = gtk.TreeView()
			store = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING) # status, name and username
			lstUsers.set_model(store)
			lstUsers.append_column(gtk.TreeViewColumn("", gtk.CellRendererPixbuf(), pixbuf=0))
			lstUsers.append_column(gtk.TreeViewColumn("", gtk.CellRendererText(), markup=1))
			colonna_username = gtk.TreeViewColumn("", gtk.CellRendererText(), text=2)
			colonna_username.set_visible(False)
			lstUsers.set_headers_visible(False)
			lstUsers.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

			# populate the list getting value from the user list in the main window
			iter = self.lstUtenti.get_model().get_iter_first()

			while not iter is None:
				lstUsers.get_model().append([self.lstUtenti.get_model().get_value(iter, 0), self.lstUtenti.get_model().get_value(iter, 1), self.lstUtenti.get_model().get_value(iter, 4)])

				iter = self.lstUtenti.get_model().iter_next(iter)

			hbox = gtk.HBox(True)

			imgScrivi = gtk.Image()
			imgScrivi.set_from_file(os.path.normpath(self.icon_theme_application_path + "message_big.png"))

			imgChiudi = gtk.Image()
			imgChiudi.set_from_file(os.path.normpath(self.icon_theme_application_path + "close.png"))

			hboxScrivi = gtk.HBox()
			hboxScrivi.pack_start(imgScrivi)
			hboxScrivi.pack_start(gtk.Label(_("Write message")))

			lblChiudi = gtk.Label(_("Close"))
			lblChiudi.set_alignment(0.0, 0.5)

			hboxChiudi = gtk.HBox()
			hboxChiudi.pack_start(imgChiudi)
			hboxChiudi.pack_start(lblChiudi)

			cmdScrivi = gtk.Button()
			cmdScrivi.add(hboxScrivi)
			cmdScrivi.connect("clicked", self.cmdMultiScrivi_clicked, win, lstUsers)

			cmdChiudi = gtk.Button()
			cmdChiudi.add(hboxChiudi)
			cmdChiudi.connect("clicked", self.cmdMultiChiudi_clicked, win)

			hbox.pack_start(cmdScrivi, False)
			hbox.pack_start(cmdChiudi, False)

			scroll.add(lstUsers)
			vbox.pack_start(scroll)
			vbox.pack_start(hbox, False)

			win.add(vbox)
			win.set_icon_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))
			win.set_title(_("Select users"))
			win.set_position(gtk.WIN_POS_MOUSE)
			win.set_default_size(300, 300)
			win.set_size_request(300, 300)
			win.show_all()
		else:
			self.message_box(_("You must be connected to send a multiple message"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winMain)

	def cmdMultiScrivi_clicked(self, widget, win, lista):
		# if there aren't selected users show a warning message
		if lista.get_selection().count_selected_rows() == 0:
			self.message_box(_("You must select at least one user"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, win)
		else:
			# otherwise call the send_message class with the users list to which send the message
			rows = lista.get_selection().get_selected_rows()

			model = rows[0]

			utenti = []
			for row in rows[1]:
				utenti.append(model.get_value(model.get_iter(row), 2))

			win.destroy() # close the window
			self.send_message_classes.append(send_message(self, utenti=utenti))

	# close the selection user window
	def cmdMultiChiudi_clicked(self, widget, win):
		win.destroy()

	# open the read sent messages window
	def mnuInviati_activate(self, widget):
		if self.count_messages(0) == 0: self.message_box(_("No message has been sent"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_read_message(0)

			if classe is None:
				self.read_message_classes.append(read_message(self, 0))
			else:
				classe.winRead.present()

	# open the received messages window
	def mnuRicevuti_activate(self, widget):
		if self.count_messages(1) == 0: self.message_box(_("No message has been received"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_read_message(1)

			if classe is None:
				self.read_message_classes.append(read_message(self, 1))
			else:
				classe.winRead.present()

	# show file transfer window
	def mnuFiletransf_activate(self, widget):
		if not self.file_manager_class is None:
			self.file_manager_class.winFileManager.show()
		else:
			self.message_box(_("No file transfer in progress"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)

	# open information window
	def mnuInfo_activate(self, widget, data=None):
		info(self)

	# open the online guide
	def mnuAiuto_activate(self, widget):
		self.open_url("http://gnu.coopi.org/pygeon/doc")

	# open launchpad to report a bug
	def mnuBug_activate(self, widget):
		self.open_url("https://bugs.launchpad.net/pygeon/+filebug")
	
	# open preferences window
	def mnuPref_activate(self, widget, data=None):
		pref(self)

	# sort contacts alphabetically
	def mnuAlphabetic_activate(self, widget):
		if self.cboStatus.get_active() != 3 and self.client.isConnected() and self.sort_method != 1: # check the user isn't offline
			self.sort_method = 1
			self.sort_roster_list()
			self.save_sort_method() # save the sort method to db
		else:
			if self.sort_method == 7:
				self.mnuSortstatus.set_active(True)

	# sort contacts by status
	def mnuSortstatus_activate(self, widget):
		if self.cboStatus.get_active() != 3 and self.client.isConnected() and self.sort_method != 7: # check the user isn't offline
			self.sort_method = 7
			self.sort_roster_list()
			self.save_sort_method() # save the sort method to db
		else:
			if self.sort_method == 1:
				self.mnuAlphabetic.set_active(True)

	# open chronology manager
	def mnuCrono_activate(self, widget):
		if self.count_chats() == 0: self.message_box(_("No conversations"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else: 
			classe = self.get_crono()

			if classe is None:
				self.crono_classes.append(crono(self))
			else:
				classe.winCrono.present()

	# crete the message box
	def message_box(self, message, title, type, buttons, parent):
		message = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, type, buttons, message)
		message.set_title(title)

		risp = message.run()
		message.destroy()

		return risp

	# create file chooser
	def file_chooser(self, title, parent, action):
		chooser = gtk.FileChooserDialog(title, parent, action)

		# add cancel button (for all type of chooser)
		chooser.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)

		# add save or open button, depending of chooser type
		if action == gtk.FILE_CHOOSER_ACTION_SAVE: chooser.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_OK)
		elif action == gtk.FILE_CHOOSER_ACTION_OPEN or gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER: chooser.add_button(gtk.STOCK_OPEN, gtk.RESPONSE_OK)

		# move to user home
		chooser.set_current_folder(os.path.expanduser('~'))

		if self.debug: print _("The file chooser window has been opened")

		risp = chooser.run()
		file = chooser.get_filename()
		chooser.destroy()
		
		if self.debug: print _("The file chooser window has been closed")

		if risp == gtk.RESPONSE_OK: return file
		else: return ""

	# save the last seen of 'utente' into db
	def save_last_seen_to_db(self, utente, last_seen):
		# convert to unicode
		u = self.tdb(utente)

		# check if is already into db
		cursor = self.conn.cursor()
		cursor.execute("select count(id) from users where user like '" + u + "%'")
		count = cursor.fetchone()[0]
		cursor.close()

		cursor = self.conn.cursor()
		if count == 0: # add if not present
			cursor.execute("insert into users (user, last_seen) values (?, ?)", (u, last_seen))
		else: # otherwise update last_seen
			cursor.execute("update users set last_seen = ? where user like '" + u + "%'", (last_seen, ))

		self.conn.commit()
		cursor.close()

		if self.debug: print _("I save the last seen of ") + utente

	# save the last seen of 'utente'
	# if 'utente' is None save the last seen of all connected users
	def save_last_seen(self, utente=None):
		# create the timestamp
		timestamp = datetime.datetime.now().strftime("%d-%m-%Y %H.%M.%S")

		# save to db
		if not utente is None:
			self.save_last_seen_to_db(utente, timestamp)
		else: # all connected users
			iter = self.lstUtenti.get_model().get_iter_first()

			while not iter is None:
				if self.lstUtenti.get_model().get_value(iter, 3) != 3:
					self.save_last_seen_to_db(self.lstUtenti.get_model().get_value(iter, 4), timestamp)

				iter = self.lstUtenti.get_model().iter_next(iter)

	# return the last seen of 'utente' if it exists
	def read_last_seen(self, utente):
		cursor = self.conn.cursor()
		cursor.execute("select last_seen from users where user like '" + self.tdb(utente) + "%'")
		last_seen = cursor.fetchone()
		cursor.close()

		if not last_seen is None:
			try:
				timestamp = last_seen[0]

				if timestamp.startswith(datetime.date.today().strftime("%d-%m-%Y")): timestamp = _("Today at ") + timestamp.split(' ')[1]
				elif timestamp.startswith((datetime.date.today() - datetime.timedelta(days=1)).strftime("%d-%m-%Y")): timestamp = _("Yesterday at ") + timestamp.split(' ')[1]
				else: timestamp = datetime.datetime.strptime(timestamp, "%d-%m-%Y %H.%M.%S").strftime(_("%d-%m-%Y %H.%M.%S"))

				return _("Last time seen: ") + timestamp
			except Exception:
				return None

		return None

	# create the pygeon preferences folder
	def create_preferences(self):
		if not os.path.isdir(self.pref_path):
			os.mkdir(self.pref_path)
			if self.debug: print _("I create the pygeon folder")
			
		if not os.path.isdir(self.pref_path + "files"):
			os.mkdir(self.pref_path + "files")
			if self.debug: print _("I create the files folder")
		
		if not os.path.isdir(self.pref_path + "avatars"):
			os.mkdir(self.pref_path + "avatars")
			if self.debug: print _("I create the avatars folder")

		# create db
		import shutil
		shutil.copy(self.current_path + "../data/db/pygeon.db", self.pref_path + "/pygeon.db")

		# connect to db
		self.conn = sqlite.connect(self.pref_path + "/pygeon.db")

		# get system language
		try:
			lang = languages_reverse[os.environ['LANG'].split('.')[0].split('_')[0]]
		except:
			lang = self.program_language

		# insert a record into preferences table
		cursor = self.conn.cursor()
		cursor.execute("insert into preferences (port, autoconnect, avatar_uploaded, files_path, accept, receive_path, receive, connect_path, connect, disconnect_path, disconnect, save_conversations, present, language, sort_method, notify_normal, notify_chat, notify_connect, notify_disconnect, notify_status, notify_add, notify_remove, notify_transfer, theme) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", (5222, 0, 0, self.tdb(os.path.normpath(self.pref_path + "files")), 2, self.tdb(os.path.normpath(self.current_path + "../data/sounds/receive.wav")), 1, self.tdb(os.path.normpath(self.current_path + "../data/sounds/connect.wav")), 0, self.tdb(os.path.normpath(self.current_path + "../data/sounds/disconnect.wav")), 0, 0, 0, self.tdb(lang), self.sort_method, 1, 1, 0, 0, 0, 1, 1, 1, self.tdb('default')))
		self.conn.commit()
		cursor.close()

	# save preferences to db reading from variables
	def save_preferences(self):	
		cursor = self.conn.cursor()
		cursor.execute("update preferences set name = ?, username = ?, password = ?, server = ?, port = ?, autoconnect = ?, avatar = ?, avatar_uploaded = ?, files_path = ?, accept = ?, receive_path = ?, receive = ?, connect_path = ?, connect = ?, disconnect_path = ?, disconnect = ?, save_conversations = ?, present = ?, language = ?, sort_method = ?, notify_normal = ?, notify_chat = ?, notify_connect = ?, notify_disconnect = ?, notify_status = ?, notify_add = ?, notify_remove = ?, notify_transfer = ?, theme = ? where id = 1", (self.tdb(self.myname), self.tdb(self.username + '@' + self.domain), base64.encodestring(self.tdb(self.password)), self.tdb(self.server), self.port, int(self.autoconnect), self.tdb(self.avatar), int(self.avatar_uploaded), self.tdb(self.files_path), int(self.files_accept), self.tdb(self.sounds_receive_path), int(self.sounds_receive), self.tdb(self.sounds_connect_path), int(self.sounds_connect), self.tdb(self.sounds_disconnect_path), int(self.sounds_disconnect), int(self.conversations_save), int(self.conversations_present), self.tdb(self.program_language), self.sort_method, int(self.notify_normal), int(self.notify_chat), int(self.notify_connect), int(self.notify_disconnect), int(self.notify_status), int(self.notify_add), int(self.notify_remove), int(self.notify_transfer), self.tdb(self.theme)))
		self.conn.commit()
		cursor.close()
		
		if self.debug: print _("Preferences saved in db from variables")

	# load preferences from db into variables
	def load_preferences(self):
		try:
			cursor = self.conn.cursor()
			cursor.execute("select * from preferences where id = 1")
			preferences = cursor.fetchone()
			cursor.close()

			self.myname = self.fdb(str(preferences[1]))

			try:
				self.username = self.fdb(str(preferences[2].split('@')[0]))
			except Exception:
				self.username = ""

			self.server = self.fdb(str(preferences[4]))

			if str(preferences[2]).find('@') > -1:
				self.domain = self.fdb(str(preferences[2])).split('@')[1]
			else:
				self.domain = self.server

			try:
				self.password = self.fdb(base64.decodestring(preferences[3]))
			except Exception:
				self.password = ""

			self.port = preferences[5]
			self.autoconnect = preferences[6] == 1
			self.avatar = self.fdb(str(preferences[7]))
			self.avatar_uploaded = preferences[8] == 1

			# files
			self.files_path = self.fdb(str(preferences[9]))
			self.files_accept = preferences[10]

			# sounds
			self.sounds_receive_path = self.fdb(str(preferences[11]))
			self.sounds_receive = preferences[12] == 1
			self.sounds_connect_path = self.fdb(str(preferences[13]))
			self.sounds_connect = preferences[14] == 1
			self.sounds_disconnect_path = self.fdb(str(preferences[15]))
			self.sounds_disconnect = preferences[16] == 1

			# notifications
			self.notify_normal = preferences[21] == 1
			self.notify_chat = preferences[22] == 1
			self.notify_connect = preferences[23] == 1
			self.notify_disconnect = preferences[24] == 1
			self.notify_status = preferences[25] == 1
			self.notify_add = preferences[26] == 1
			self.notify_remove = preferences[27] == 1
			self.notify_transfer = preferences[28] == 1

			# conversations
			self.conversations_save = preferences[17]
			self.conversations_present = preferences[18] == 1

			# language
			self.program_language = self.fdb(str(preferences[19]))

			if os.path.isdir(os.path.normpath(self.current_path + "../data/images/themes/" + str(preferences[29]))):
				self.theme = self.fdb(str(preferences[29]))
			else:
				self.theme = "default"

			# sort_method
			self.sort_method = preferences[20]

			if self.debug: print _("Preferences loaded in the variables from db")
		except Exception:
			pass

	# change the status (0=available, 1=away, 2=not available, 3=offline) from the combo and tray icon
	def change_status(self, status):
		if status == 0:
			# create the jabber connection
			if self.client is None or not self.client.isConnected():
				if self.jabber_connect(): # if the connection is ok go on
					self.tray.tray.set_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))
					self.tray.tray.set_tooltip(_("Pygeon - Available"))
					self.presence_message = "" # empty the presence message
				else: self.cboStatus.set_active(3) # otherwise set the combo to offline status
			else: # if the user it's already connected change icons
				self.presence_message = "" # empty the presence message
				self.send_presence(status) # send the presence message without open the window
				if not self.tray.tray.get_blinking():
					self.tray.tray.set_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))
					self.tray.tray.set_tooltip(_("Pygeon - Available"))
		elif status == 1:
			# only if the user is already connected
			if self.client is None or not self.client.isConnected():
				self.cboStatus.set_active(3)
				return

			self.create_status_window() # open the window for status message
			if not self.tray.tray.get_blinking():
				self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "away.png"))
				self.tray.tray.set_tooltip(_("Pygeon - Away"))
		elif status == 2:
			# only if the user is already connected
			if self.client is None or not self.client.isConnected():
				self.cboStatus.set_active(3)
				return

			self.create_status_window() # open the window for status message 
			if not self.tray.tray.get_blinking():
				self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "dnd.png"))
				self.tray.tray.set_tooltip(_("Pygeon - Not available"))
		elif status == 3:
			# save last seen of connected users
			self.save_last_seen()

			if not self.network_down: # if the network isn't down
				if not self.tray.tray.get_blinking():
					self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png"))
					self.tray.tray.set_tooltip(_("Pygeon - Offline"))

				# if the client is connected disconnect it
				if not self.client is None and self.client.isConnected():
					self.jabber_disconnect()
					self.close_all_chats() # close all chat windows
			else: # if the network is down
				self.network_down = False # reset the variable

				self.lstUtenti.get_model().clear() # empty contact list
				self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png"))
				self.tray.tray.set_tooltip(_("Pygeon - Offline"))

				# start a timer that try to reconnect the client (it acts only one time)
				gobject.timeout_add(60000, self.try_reconnect)		
				

	# when the combo change selection
	def cboStatus_changed(self, widget):
		self.change_status(self.cboStatus.get_active())

	# listen jabber messages from the server
	def jabber_listen(self):
		while self.client.Process(0):
			while self.client.Process(0) != '0':
				pass
			yield True
		yield False

	# iq message received from the jabber server
	def receive_iq(self, client, message):
		if self.debug: print message

		if message.getType() == "set": # add/remove user
			if not message.getTag('query') is None:
				# add/remove user
				query = message.getTag('query')

				for item in query.getChildren(): # scroll all items
					subscription = item.getAttr('subscription')

					if subscription == "both": # user to add to the list
						self.lstUtenti.get_model().append([gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png")), item.getAttr('name'), None, 3, item.getAttr('jid'), 0, self.read_last_seen(item.getAttr('jid')), "3" + item.getAttr('name'), 0])

						# show notification
						self.show_notification("subscription", "", _("Add to contact list"), item.getAttr('name') + " " + _("has been added to contact list"))
						self.count_rosters()
					elif subscription == "remove": # user to remove from the list
						# this trunk is managed by receive_presence
						pass
		elif message.getType() == "result":
			# receive all rosters
			if not message.getTag("query") is None and str(message.getTag("query").getNamespace()) == "jabber:iq:roster":
				query = message.getTag('query')

				for item in query.getChildren(): # add all items into the contact list
					# check if avatar must be added
					if os.path.isfile(os.path.normpath(self.pref_path + "/avatars/" + str(item.getAttr('jid')) + "_small")):
						avatar = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.pref_path + "/avatars/" + str(item.getAttr('jid')) + "_small"))
					else:
						avatar = None

					self.lstUtenti.get_model().append([gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png")), item.getAttr('name'), avatar, 3, item.getAttr('jid'), 0, self.read_last_seen(item.getAttr('jid')), "3" + item.getAttr('name'), 0])

				self.sort_roster_list() # sort contacts
			elif not message.getTag("vCard") is None and str(message.getTag("vCard").getNamespace()) == "vcard-temp": # roster avatar
				vcard = message.getTag('vCard')

				if self.debug: print _("I received an avatar from ") + str(message.getFrom())

				photo = None

				# try to get the photo
				for child in vcard.getChildren():
					if child.getName() == "PHOTO":
						photo = child
						break

				# if the photo is present try to get type and binval
				if not photo is None:
					if self.debug: print _("I found the tag PHOTO")

					type = None
					binval = None

					for child in photo.getChildren():
						if child.getName() == "TYPE": type = child.getData()
						elif child.getName() == "BINVAL": binval = child.getData()

					# save the photo if all data is found
					if not type is None and (type == "image/png" or type == "image/jpeg") and not binval is None:
						self.save_avatar(binval, type, message.getFrom())
						if self.debug: print _("I save the picture of ") + str(message.getFrom())
			else: # the server respond ok to the vcard
				# send a presence message with the avatar hash
				if self.debug: print _("I received the ok form the server to upload my avatar")

				# save the avatar into the pref variable getting it from the temporary var
				if self.avatar_temp == os.path.normpath(os.path.normpath(self.icon_theme_application_path + "no_avatar.png")):
					self.avatar_temp = ""
 
				self.avatar = self.avatar_temp
				self.avatar_temp = ""

				# save the avatar to db
				cursor = self.conn.cursor()
				cursor.execute("update preferences set avatar = ? where id = 1", (self.tdb(self.avatar),))
				self.conn.commit()
				cursor.close()

				# send the message of change avatar to all users
				self.send_presence(self.cboStatus.get_active())

	# create the avatar tag to send with a presence message
	def create_avatar_tag(self):
		x = xmpp.simplexml.Node("vcard-temp:x:update x")
		hash = self.get_avatar_hash(self.avatar)

		if hash == "":
			photo = xmpp.simplexml.Node("photo")
		else:
			photo = xmpp.simplexml.Node("photo", {}, hash)

		x.addChild(node = photo)

		if self.ddebug: print _("I created the tag for avatar: ") + str(x)

		return x

	# display a notification popup
	def show_notification(self, tipo, user, title, message):
		if not self.just_connected:
			# go on if notification is enabled
			if (tipo == "normal" and self.notify_normal) or (tipo == "chat" and self.notify_chat) or (tipo == "online" and self.notify_connect) or (tipo == "unavailable" and self.notify_disconnect) or ((tipo == "away" or tipo == "dnd" or tipo == "available") and self.notify_status) or (tipo == "subscription" and self.notify_add) or (tipo == "unsubscribe" and self.notify_remove) or (tipo == "file-transfer" and self.notify_transfer):

				if title is None or title == "": title = _("No subject")
	
				if self.debug:
					print _("I call notification of type: ") + tipo + _(", from: ") + user + _(", with title: ") + title + _(", with message: ") + message

				# choose the image
				if tipo != "subscription" and tipo != "unsubscribe" and os.path.isfile(os.path.normpath(self.pref_path + "/avatars/" + user.split('/')[0])):
					image = os.path.normpath(self.pref_path + "/avatars/" + user.split('/')[0])
				elif tipo == "chat" or tipo == "normal":
					image = os.path.normpath(self.icon_theme_application_path + "new_message.png")
				else:
					image = os.path.normpath(self.icon_application_path + "notification.png")

				self.notifier.new_popup(pygeon=self, user=self.get_name_from_username(user.split('/')[0]), title=title, message=message, tipo=tipo, image=image)

	# check if dir 'path' is writeable
	def check_dir_writable(self, path):
		try:
			f = open(os.path.normpath(path + "/ttmp"), 'w')
			f.write('ttmp')
			f.close()
			os.remove(os.path.normpath(path + "/ttmp"))

			return True
		except:
			return False
		
	# receive message from jabber server
	def receive_message(self, client, message):
		if self.debug: print str(message)

		tipo = message.getType()

		if tipo == "file-transfer": # file transfer request
			if not message.getTag('file-transfer') is None:
				if self.debug: _("There is a file transfer request")

				mittente = str(message.getAttr('from'))
				fid = str(message.getAttr('id'))
				ft = message.getTag('file-transfer')
				filename = str(ft.getAttr('filename'))

				# check the destination directory exists and is writeable
				if self.files_accept != 1 and not os.path.isdir(os.path.normpath(self.files_path)):
					# if dir doesn't exist refuse the file
					self.send_file_transfer_rejected(to=mittente, fid=fid)

					# error message
					self.message_box(_("Transfer of ") + filename + _(" from ") + self.get_name_from_username(mittente.split('/')[0]) + _(" has been refused because the destination folder doesn't exists. Check the preferences"), _("File transfer refused"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)

					if self.debug: print _("The destination folder doesn't exists. I refused the file.")
				elif os.path.isdir(os.path.normpath(self.files_path)):
					# check the folder is writeable
					if not self.check_dir_writable(os.path.normpath(self.files_path)):
						# refuse the file if the directory isn't writeable
						self.send_file_transfer_rejected(to=mittente, fid=fid)

						# error message
						self.message_box(_("Transfer of ") + filename + _(" from ") + self.get_name_from_username(mittente.split('/')[0]) + _(" has been refused because the destination folder is not writeable. Check the destination folder or change it from preferences window."), _("File transfer refused"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)

						if self.debug: print _("The destination folder is not writeable. I refused the file.")
					else:
						# check preferences
						if self.files_accept == 0 or (self.files_accept == 2 and self.message_box(self.get_name_from_username(mittente.split('/')[0]) + _(" want to send you the file ") + filename + _(". Do you want to accept the file?"), _("Receiving file"), gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, self.winMain) == gtk.RESPONSE_YES):
							# accept file
							host = str(message.getAttr('host'))
							size = int(str(ft.getAttr('size')))

							# create the file_manager class if it doesn't exist yet
							if self.file_manager_class is None:
								self.file_manager_class = file_manager(self)

							# call a file_transfer class to open a socket
							self.file_transfers.append(file_transfer(self, filename=filename, mittente=mittente, fid=fid, size=size))

							if self.debug: print _("I add a file transfer class. Now the list size is: ") + str(len(self.file_transfers))
						else:
							# refuse the file and send a reject message
							self.send_file_transfer_rejected(to=mittente, fid=fid)

							if self.debug: print _("I refused the file")
		elif tipo == "file-transfer-accepted": # file transfer accepted
			if not message.getTag('file-transfer') is None:
				if self.debug: print _("File transfer has been accepted. I connect through socket.")

				ft = message.getTag('file-transfer')
				fid = str(message.getAttr('id'))
				host = str(ft.getAttr('host'))
				port = int(str(ft.getAttr('port')))

				# set into file_transfer class host and port
				self.file_transfers[self.get_file_transfer(fid)].host = host
				self.file_transfers[self.get_file_transfer(fid)].port = port
				self.file_transfers[self.get_file_transfer(fid)].accepted = True

				# if the file_manager class doesn't exist create it
				if self.file_manager_class is None:
					self.file_manager_class = file_manager(self)
		elif tipo == "file-transfer-rejected": # file transfer rejected
			fid = str(message.getAttr('id'))

			# tell file transfer class file transfer rejecting
			self.file_transfers[self.get_file_transfer(fid)].accepted = False
		elif tipo == "file-transfer-finished": # file transfer finished
			fid = str(message.getAttr('id'))
			
			# tell file transfer class file transfer is finished
			self.file_transfers[self.get_file_transfer(fid)].finished = True
		elif tipo == "file-transfer-cancelled": # file transfer cancelled
			fid = str(message.getAttr('id'))

			# tell file transfer class file transfer has been cancelled
			self.file_transfers[self.get_file_transfer(fid)].cancelled = True
		elif tipo == "file-transfer-ok": # file transfer ok
			destinatario = self.get_name_from_username(str(message.getAttr('from')).split('/')[0])
			filename = str(message.getAttr('filename'))

			# show notification
			self.show_notification("file-transfer", destinatario, _("File transfer finished"), _("Sending of ") + filename + _(" to ") + destinatario + _(" was successfully"))

			if self.debug: print _("File transfer was successfully")
		elif tipo == "file-transfer-error": # file transfer failed
			destinatario = self.get_name_from_username(str(message.getAttr('from')).split('/')[0])
			filename = str(message.getAttr('filename'))

			# show notification and error message
			self.show_notification("file-transfer", destinatario, _("File transfer failed"), _("Sending of ") + filename + _(" to ") + destinatario + _(" is failed"))

			self.message_box(_("Transfer of ") + filename + _(" to ") + destinatario + _(" is failed"), _("File transfer failed"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)
			
			if self.debug: print _("File transfer is failed")
		elif tipo == "normal": # normal message
			self.save_unread_message(message)

			# if the function that counts messages has already act then increment the new messages counter
			if self.message_counter:
				self.add_message_to_contact(message)

			user = str(message.getFrom())
			
			title = None
			for item in message.getChildren():
				if item.getName() == "subject": title = str(item.getData())
				if item.getName() == "body": body = str(item.getData())

			# show notification
			self.show_notification(tipo, user, title, body)
		elif tipo == "chat": # chat message
			try:
				body = str(message.getTag('body').getData())
				thread = str(message.getTag('thread').getData())
				utente = str(message.getFrom()).split('/')[0]

				if self.debug: print _("Received the chat message with body: ") + body + _(" and thread: ") + thread

				# if a chat with the specified thread exists use it, otherwise open a new chat window
				position = self.get_chat_from_thread(thread)
				if position == -1: # create a new chat window
					self.chats.append(chat(self, utente, thread, body))
				
					# if the chat window isn't visible increment the chat counter in the contact list
					if not self.chats[len(self.chats) - 1].winChat.get_property("visible"):
						self.add_chat_to_contact(str(message.getFrom()))

						# show notification
						self.show_notification(tipo, str(message.getFrom()), thread, body)
				else: # append the message to the existing window
					# add timestamp
					self.add_timestamp_to_chat(self.chats[position].txtBody, self.get_name_from_username(utente), "timestamp_other")

					# get offset
					offset = self.chats[position].txtBody.get_buffer().get_char_count()

					# add message into textview
					self.chats[position].txtBody.get_buffer().insert(self.chats[position].txtBody.get_buffer().get_end_iter(), body)

					# map all url message
					self.add_url_to_message(self.chats[position].txtBody, offset)

					# add emoticons to message
					self.add_emoticons_to_message(self.chats[position].txtBody, offset)

					# scroll to end
					self.chats[position].txtBody.scroll_to_mark(self.chats[position].txtBody.get_buffer().create_mark(None, self.chats[position].txtBody.get_buffer().get_end_iter(), False), 0.0)

					if not self.chats[position].winChat.get_property("visible"): # if the window is not visible
						# show notification
						self.show_notification(tipo, str(message.getFrom()), thread, body)
					elif not self.chats[position].winChat.is_active(): # if the window is visible and not active
						# show notification
						self.show_notification(tipo, str(message.getFrom()), thread, body)
						
						# activate urgency-hint
						self.chats[position].winChat.set_urgency_hint(True)
			except Exception:
				if self.debug: print _("Wrongly built message.. I don't consider it")

		# sounds
		if (message.getType() == "normal" or message.getType() == "chat") and not self.mute and self.sounds_receive: self.play_sound(self.sounds_receive_path)

	# save received and unread messages 
	def save_unread_message(self, message):
		# get sender
		msg_from = str(message.getFrom())
		stamp = None

		msg_subject = ""

		# get subject, body and timestamp if present
		for item in message.getChildren():
			if item.getName() == "subject": msg_subject = str(item.getData())
			if item.getName() == "body": msg_body = str(item.getData())
			if item.getName() == "x" and stamp is None: stamp = item.getAttr('stamp') # get the first stamp in the message

		# if there isn't subject put a default subject
		if msg_subject == "": msg_subject = _("No subject")
		
		# parse timestamp if available
		if not stamp is None:
			if len(stamp.split('T')[0]) < 10: stamp = stamp[:4] + "-" + stamp[4:6] + "-" + stamp[6:]
			msg_name = stamp.replace(':', '-')
			msg_date = datetime.datetime.strptime(msg_name.split('.')[0], "%Y-%m-%dT%H-%M-%S").strftime("%d-%m-%Y %H.%M.%S")
			if self.debug: print _("I use the message timestamp")
		else: # otherwise create a new timestamp
			msg_name = datetime.datetime.now().isoformat().replace(':', '-')
			msg_date = datetime.datetime.strptime(msg_name.split('.')[0], "%Y-%m-%dT%H-%M-%S").strftime("%d-%m-%Y %H.%M.%S")
			if self.debug: print _("I create a timestamp for the message")

		cursor = self.conn.cursor()
		cursor.execute("insert into messages (date, with, subject, body, type) values (?, ?, ?, ?, 2)", (msg_date, self.tdb(msg_from), self.tdb(msg_subject.decode('utf-8')), self.tdb(msg_body.decode('utf-8'))))
		self.conn.commit()
		cursor.close()

	# increment new chat counter for 'utente'
	def add_chat_to_contact(self, utente):
		utente = utente.split('/')[0]

		# search for the user
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == utente:
				count = self.lstUtenti.get_model().get_value(iter, 8)
				self.lstUtenti.get_model().set_value(iter, 8, count + 1)

				# change icon if necessary
				if self.lstUtenti.get_model().get_value(iter, 5) == 0 and count == 0:
					self.lstUtenti.get_model().set_value(iter, 0, gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_application_path + "unread_messages.png")))

				if self.debug: print _("I increased the chat counter of ") + utente + _(". Now is: ") + str(count+1)

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# remove a new chat from the counter for 'utente'
	def remove_chat_from_contact(self, utente):
		utente = utente.split('/')[0]

		# search for the user
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == utente:
				count = self.lstUtenti.get_model().get_value(iter, 8)
				self.lstUtenti.get_model().set_value(iter, 8, count - 1)

				# if there aren't new messages/chats change icon
				if count == 1 and self.lstUtenti.get_model().get_value(iter, 5) == 0:
					self.change_roster_status(utente, self.lstUtenti.get_model().get_value(iter, 3), None, True)

				if self.debug: print _("I have decreased the chat counter of ") + utente + _(". Now is: ") + str(count-1)

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# add a new message to 'utente' if specified, otherwise get the user from 'message'
	def add_message_to_contact(self, message, utente=None):
		if not utente is None: utente = utente.split('/')[0]
		else: utente = str(message.getFrom()).split('/')[0]

		# search for the user
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == utente:
				count = self.lstUtenti.get_model().get_value(iter, 5)
				self.lstUtenti.get_model().set_value(iter, 5, count + 1)

				# if there aren't new messages/chats change icon
				if self.lstUtenti.get_model().get_value(iter, 8) == 0 and count == 0: self.lstUtenti.get_model().set_value(iter, 0, gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_application_path + "unread_messages.png")))

				if self.debug: print _("I incremented the message counter of ") + utente + _(". Now is: ") + str(count+1)
				
				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# remove a new message from 'utente'
	# if clear is False reset the counter
	def remove_message_from_contact(self, utente, clear=False):
		utente = str(utente).split('/')[0]

		# search for the user
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == utente:
				count = self.lstUtenti.get_model().get_value(iter, 5)

				if not clear: 
					self.lstUtenti.get_model().set_value(iter, 5, count - 1)
				else:
					self.lstUtenti.get_model().set_value(iter, 5, 0)

				# if there aren't new messages/chats change icon
				if (count == 1 or clear) and self.lstUtenti.get_model().get_value(iter, 8) == 0: self.change_roster_status(utente, self.lstUtenti.get_model().get_value(iter, 3), None, True)

				if self.debug:
					if not clear: print _("I have decreased the counter of ") + utente + _(". Now is: ") + str(count-1)
					else: print _("I reset the counter of ") + utente

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# check for unread messages and set the counter
	def load_message_counter(self):
		cursor = self.conn.cursor()
		cursor.execute("select with from messages where type = 2")
		
		for user in cursor:
			self.add_message_to_contact(None, self.fdb(user[0]))

		cursor.close()
			
		self.message_counter = True
		return False

	# save the sent message into db
	def save_sent_message(self, msg_date, msg_to, msg_subject, msg_body):
		cursor = self.conn.cursor()
		cursor.execute("insert into messages (date, with, subject, body, type) values (?, ?, ?, ?, 0)", (msg_date, self.tdb(msg_to), self.tdb(msg_subject.decode('utf-8')), self.tdb(msg_body.decode('utf-8'))))
		self.conn.commit()
		cursor.close()

	# set just_connected to False after 10 seconds
	def set_just_connected(self):
		self.just_connected = False
		
		if self.debug: print _("I set just_connected to False")

		return False # stop timer

	# set tray tooltip
	def set_tray_tooltip(self, mex_count, chats_count):
		if mex_count == 1 and chats_count == 0: self.tray.tray.set_tooltip(_("Pygeon - 1 new message"))
		elif mex_count == 1 and chats_count == 1: self.tray.tray.set_tooltip(_("Pygeon - 1 new message and 1 new chat"))
		elif mex_count == 1 and chats_count > 1: self.tray.tray.set_tooltip(_("Pygeon - 1 new message and") + " " + str(chats_count) + _(" new chats"))
		elif mex_count == 0 and chats_count == 1: self.tray.tray.set_tooltip(_("Pygeon - 1 new chat"))
		elif mex_count == 0 and chats_count > 1: self.tray.tray.set_tooltip(_("Pygeon - ") + str(chats_count) + _(" new chats"))
		elif mex_count > 1 and chats_count == 0: self.tray.tray.set_tooltip(_("Pygeon - ") + str(mex_count) + _(" new messages"))
		elif mex_count > 1 and chats_count == 1: self.tray.tray.set_tooltip(_("Pygeon - ") + str(mex_count) + _(" new messages and 1 new chat"))
		elif mex_count > 1 and chats_count > 1: self.tray.tray.set_tooltip(_("Pygeon - ") + str(mex_count) + _(" new messages and") + " " + str(chats_count) + _(" new chats"))

	# check if there are unread messages or new chats and manager the tray icon blinking
	def check_unread_messages(self):
		if not self.check_unread:
			self.check_unread = True # semaphore

			mex_count = self.count_messages(2) # unread messages counter
			chats_count = self.count_new_chats() # new chats counter

			if (mex_count > 0 or chats_count > 0) and not self.tray.tray.get_blinking():
				# if there are unread messages or new chats and the tray don't blink
				self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_application_path + "new_message_small.png"))
				self.set_tray_tooltip(mex_count, chats_count)	
			
				self.tray.tray.set_blinking(True)
			elif mex_count == 0 and chats_count == 0 and self.tray.tray.get_blinking():
				# if there aren't new messages and new chats and the tray blink
				self.tray.tray.set_blinking(False)

				# return to previuos status (don't consider offline because it will be managed after)
				if self.cboStatus.get_active() == 0:
					self.tray.tray.set_tooltip(_("Pygeon - Available"))
					self.tray.tray.set_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))
				elif self.cboStatus.get_active() == 1:
					self.tray.tray.set_tooltip(_("Pygeon - Away"))
					self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "away.png"))
				elif self.cboStatus.get_active() == 2:
					self.tray.tray.set_tooltip(_("Pygeon - Not available"))
					self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "dnd.png"))
			elif (mex_count + chats_count > 0) and mex_count + chats_count != self.unread_messages_count:
				# if there are new messages or new chats and it's blinking
				self.unread_messages_count = mex_count + chats_count
				self.set_tray_tooltip(mex_count, chats_count)	

			self.check_unread = False # disable semaphore
		
			# always return True until the user disconnect from the server
			if self.cboStatus.get_active() != 3: return True
			else:
				self.tray.tray.set_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png"))
				self.tray.tray.set_tooltip(_("Pygeon - Offline"))
				self.tray.tray.set_blinking(False)

				return False

		else: return True

	# try to reconnect and stop
	def try_reconnect(self):
		if self.debug: print _("I try to make an automatic reconnection to the server...")

		# move combo on available status to start jabber_connect
		self.reconnecting = True
		self.cboStatus.set_active(0)

		return False

	# check if the network is ok
	def check_network(self):
		if not self.client.isConnected() and self.cboStatus.get_active() != 3:
			if self.debug: print _("The network is down... I start the disconnection")
			
			self.network_down = True	
			self.cboStatus.set_active(3)

			return False
		elif not self.client.isConnected():
			return False
		else: 
			# set the check time
			self.previous_check = time.time()

			return True

	# return the read_message class 
	def get_read_message(self, type, utente=None):
		for classe in self.read_message_classes:
			if classe.type == type:
				if (utente is None and classe.utente is None) or (not utente is None and classe.utente == utente):
					return classe

		return None

	# return the chronology class
	def get_crono(self, utente=None):
		for classe in self.crono_classes:
			if (utente is None and classe.utente is None) or (not utente is None and classe.utente == utente):
				return classe

		return None

	# return the number of messages of type 'type' of user 'utente'
	# if 'utente' is None don't apply the filter
	def count_messages(self, type, utente=None):
		cursor = self.conn.cursor()

		if utente is None:
			cursor.execute("select count(id) from messages where type = " + str(type))
		else:
			cursor.execute("select count(id) from messages where type = " + str(type) + " and with like '" + self.tdb(utente) + "%'")

		return cursor.fetchone()[0]

	# return the number of new conversations (not visible yet)
	def count_new_chats(self):
		count = 0

		for classe in self.chats:
			if not classe.winChat.get_property("visible"):
				count += 1

		return count
		
	# return the number of conversations of user 'utente', or all if 'utente' is None
	def count_chats(self, utente=None):
		cursor = self.conn.cursor()
		
		if utente is None:
			cursor.execute("select count(id) from chats")
		else:
			cursor.execute("select count(id) from chats where with like '" + self.tdb(utente) + "%'")

		return cursor.fetchone()[0]
		
	# get the status name from the status type
	def get_status_from_type(self, type):
		if type == "away" or type == "xa": return _("Away")
		elif type == "dnd": return _("Not available")
		elif type == "unavailable": return _("Offline")
		else: return _("Available")

	# get the status name from number
	def get_status_from_number(self, number):
		if number == 0: return _("Available")
		elif number == 1: return _("Away")
		elif number == 2: return _("Not available")

		return _("Offline")

	# get the status of 'utente'
	def get_user_status(self, utente):
		iter = self.lstUtenti.get_model().get_iter_first()

		status = None

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == utente:
				status = self.lstUtenti.get_model().get_value(iter, 3)

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

		return status

	# presence messages received from the server
	def receive_presence(self, client, message):
		tipo = message.getType()
		da = message.getFrom()
		info = None
		
		if self.debug:
			print _("Messagge from ") + str(da) + _(" of type ") + str(tipo)
			print str(message)

		if str(tipo) == "unavailable": # offline
			# save its last seen if the user isn't disconnected yet
			if self.get_user_status(str(da).split('/')[0]) != 3:
				self.save_last_seen(str(da).split('/')[0])

				# show notification
				self.show_notification(str(tipo), str(da), _("Offline"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is offline"))

				if not self.mute and self.sounds_disconnect:
					self.play_sound(self.sounds_disconnect_path)

			self.change_roster_status(da, 3)
			self.count_rosters()
		elif str(tipo) == "unsubscribe": # user doesn't exist yet
			# show notification
			self.show_notification(str(tipo), str(da), _("Remove from contact list"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("has been removed from contact list"))

			# remove user from list
			self.remove_roster_from_list(str(da).split('/')[0])
			self.count_rosters()			
		elif str(tipo) == "unsubscribed": # user doesn't exist yet (confirm)
			pass # doesn't do anything
		else:
			tipo = message.getShow() # get the status
			info = message.getStatus() # additional information on status

			if not info is None and self.debug: print _("Info of ") + str(da) + ": " + str(info)

			if tipo == "away" or tipo == "xa": # away and not available => away
				# show notification
				if self.get_user_status(str(da).split('/')[0]) != 1:
					if info is None:
						self.show_notification("away", str(da), self.get_name_from_username(str(da).split('/')[0]) + " " + _(" is away"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is away"))
					else:
						self.show_notification("away", str(da), self.get_name_from_username(str(da).split('/')[0]) + " " + _(" is away"), info)
				
				self.change_roster_status(da, 1, info)
			elif tipo == "dnd": # don't disturb => not available
				# show notification
				if self.get_user_status(str(da).split('/')[0]) != 2:
					if info is None:
						self.show_notification(tipo, str(da), self.get_name_from_username(str(da).split('/')[0]) + " " + _(" is Not available"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is Not available"))
					else:
						self.show_notification(tipo, str(da), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is Not available"), info)
				
				self.change_roster_status(da, 2, info)
			else: # available
				# sounds
				if self.get_user_status(str(da).split('/')[0]) == 3 and not self.mute and self.sounds_connect:
					self.play_sound(self.sounds_connect_path)

				# show notification
				if self.get_user_status(str(da).split('/')[0]) == 3: # connected
					self.show_notification("online", str(da), _("Available"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is available"))
				elif self.get_user_status(str(da).split('/')[0]) != 0: # is available
					self.show_notification("available", str(da), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is available"), self.get_name_from_username(str(da).split('/')[0]) + " " + _("is available"))

				self.change_roster_status(da, 0, None, False)
				self.count_rosters()

		# notify status change in chat windows
		self.add_status_to_chat(str(da).split('/')[0], self.get_status_from_type(tipo), info)

		# check if there is an avatar (doesn't check if it is a disconnection or deletion)
		if str(tipo) != "unavailable" and str(tipo) != "unsubscribe" and str(tipo) != "unsubscribed":
			hash = None
			tag = message.getTag('x')
		
			if not tag is None:
				# check if the namespace is correct
				if tag.getNamespace() == "vcard-temp:x:update":
					for child in tag.getChildren():
						if child.getName() == "photo":
							hash = child.getData()
							break

			if not hash is None and hash != "": # check hash
				local_hash = self.get_avatar_hash(os.path.normpath(self.pref_path + "/avatars/" + str(da).split('/')[0]))

				if self.ddebug: print _("I compare:\n") + hash + _("with:\n") + local_hash

				if hash != local_hash: # if hashes are different request the new avatar
					if self.ddebug: print _("Hash do not match")
					self.send_iq(str(da).split('/')[0], "get", "vcard-temp")
			else:
				# remove the avatar in the list if present
				self.remove_avatar_from_user(str(da).split('/')[0])

				# delete files
				if os.path.isfile(os.path.normpath(self.pref_path + "/avatars/" + str(da).split('/')[0])):
					os.remove(os.path.normpath(self.pref_path + "/avatars/" + str(da).split('/')[0]))

				if os.path.isfile(os.path.normpath(self.pref_path + "/avatars/" + str(da).split('/')[0] + "_small")):
					os.remove(os.path.normpath(self.pref_path + "/avatars/" + str(da).split('/')[0] + "_small"))

	# add avatar to contact list
	def add_avatar_to_user(self, utente):
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4).startswith(utente):
				self.lstUtenti.get_model().set_value(iter, 2, gtk.gdk.pixbuf_new_from_file(self.pref_path + "/avatars/" + utente + "_small"))
				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# remove the avatar from contact list
	def remove_avatar_from_user(self, utente):
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4).startswith(utente) and not self.lstUtenti.get_model().get_value(iter, 2) is None:
				self.lstUtenti.get_model().set_value(iter, 2, None)

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

	# return the hash of 'filename'
	def get_avatar_hash(self, filename):
		if os.path.isfile(filename):
			import hashlib

			f = open(filename, 'rb')
			hash = hashlib.sha1(base64.encodestring(f.read())).hexdigest()
			f.close()

			return hash
		else:
			return ""

	# save the avatar in the avatar folder
	def save_avatar(self, data, type, utente):
		while gtk.events_pending(): gtk.main_iteration()

		try:
			# create the avatar folder if doesn't exist
			if not os.path.isdir(os.path.normpath(self.pref_path + "/avatars")):
				os.mkdir(os.path.normpath(self.pref_path + "/avatars"))

			# save the avatar
			if not data is None and data != "":
				f = open(os.path.normpath(self.pref_path + "/avatars/" + str(utente)), 'wb')
				f.write(base64.decodestring(data))
				f.close()

				# create also the thumb for the contact list
				pix = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.pref_path + "/avatars/" + str(utente)))
				pix_small = pix.scale_simple(30, 30, gtk.gdk.INTERP_BILINEAR);
				pix_small.save(os.path.normpath(self.pref_path + "/avatars/" + str(utente)) + "_small", type.split('/')[1])

				# load the avatar into the contact list
				self.add_avatar_to_user(str(utente))
			else:
				if self.debug: print _("The avatar saving failed. Empty BINVAL")
		except Exception:
			if self.debug: print _("The avatar saving failed. Empty BINVAL")

	# create the window to insert a description of the status
	def create_status_window(self):
		self.winStatus = gtk.Window()

		vbox = gtk.VBox()
		
		label = gtk.Label()
		label.set_use_markup(True)
		label.set_markup(_("<b>Insert an optional custom message</b>"))

		self.txtStatus = gtk.Entry()

		hbox = gtk.HBox()
			
		imgOk = gtk.Image()
		imgOk.set_from_file(os.path.normpath(self.icon_theme_application_path + "ok.png"))

		hboxOk = gtk.HBox()
		hboxOk.pack_start(imgOk)
		hboxOk.pack_start(gtk.Label(_("Ok")))
		
		self.cmdStatus = gtk.Button()
		self.cmdStatus.add(hboxOk)
		self.cmdStatus.set_flags(gtk.CAN_DEFAULT)
		self.cmdStatus.set_flags(gtk.HAS_DEFAULT)
		self.txtStatus.set_activates_default(True)

		hbox.pack_end(self.cmdStatus, False)
		
		vbox.pack_start(label)
		vbox.pack_start(self.txtStatus)
		vbox.pack_start(hbox)

		# signals
		self.cmdStatus.connect("clicked", self.cmdStatus_clicked)
		self.winStatus.connect("delete_event", self.winStatus_delete)

		self.winStatus.add(vbox)
		self.winStatus.set_modal(True)
		self.winStatus.set_title(_("Change status"))
		self.winStatus.set_icon_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))

		self.cmdStatus.grab_default()

		self.winStatus.show_all()

	# click on OK button
	def cmdStatus_clicked(self, widget):
		self.presence_message = self.txtStatus.get_text() # get the message
		self.winStatus.destroy() # destroy the window

		self.send_presence(self.cboStatus.get_active()) # send presence message

	# window closing
	def winStatus_delete(self, widget, event):
		self.presence_message = "" # empty the presence message
		self.winStatus.destroy() # destroy the window

		self.send_presence(self.cboStatus.get_active()) # send the presence message

	# send a file transfer request
	def send_file_transfer_request(self, to, filename, fid, size):
		message = xmpp.protocol.Message(typ="file-transfer", to=to)
		message.setAttr('id', fid)
		file_transfer = message.setTag('file-transfer')
		file_transfer.setAttr('filename', filename)
		file_transfer.setAttr('size', size)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer accepted message
	def send_file_transfer_accepted(self, to, host, port, fid):
		message = xmpp.protocol.Message(typ="file-transfer-accepted", to=to)
		message.setAttr('id', fid)
		file_transfer = message.setTag('file-transfer')
		file_transfer.setAttr('host', host)
		file_transfer.setAttr('port', port)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer rejected message
	def send_file_transfer_rejected(self, to, fid):
		message = xmpp.protocol.Message(typ="file-transfer-rejected", to=to)
		message.setAttr('id', fid)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer finished message
	def send_file_transfer_finished(self, to, fid):
		message = xmpp.protocol.Message(typ="file-transfer-finished", to=to)
		message.setAttr('id', fid)
		
		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer cancelled message
	def send_file_transfer_cancelled(self, to, fid):
		message = xmpp.protocol.Message(typ="file-transfer-cancelled", to=to)
		message.setAttr('id', fid)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer ok message
	def send_file_transfer_ok(self, to, fid, filename):
		message = xmpp.protocol.Message(typ="file-transfer-ok", to=to)
		message.setAttr('id', fid)
		message.setAttr('filename', filename)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send a file transfer error message
	def send_file_transfer_error(self, to, fid, filename):
		message = xmpp.protocol.Message(typ="file-transfer-error", to=to)
		message.setAttr('id', fid)
		message.setAttr('filename', filename)

		self.client.send(message)

		if self.debug: print "\n" + str(message)

	# send an iq message
	def send_iq(self, to, type, query, payload=[]):
		if self.client.isConnected(): # send message only if the user is connected
			self.client.send(xmpp.protocol.Iq(type, query, {}, to, self.jid, payload))
			if self.debug: print _("I sent an iq message of type: ") + str(type) + _(" to ") + str(to) + _(" to ask ") + str(query)

	# send a presence message
	def send_presence(self, status):
		if self.client.isConnected(): # send message only if the user is connected
			# get status
			if status == 0: show = None # available
			elif status == 1: show = "away" # away
			elif status == 2: show = "dnd" # not available

			if self.presence_message == "": message = None
			else: message = self.presence_message

			# if there is an avatar send also the file hash
			if self.avatar is None:
				self.client.send(xmpp.protocol.Presence(None, None, None, show, message))
			else:
				self.client.send(xmpp.protocol.Presence(None, None, None, show, message, {}, self.jid, None, [self.create_avatar_tag()]))

			if self.debug: print _("I sent the presence status: ") + str(show) + _(" with message: ") + str(message)

	# send a message
	def send_message(self, to, type, subject, body):
		if self.client.isConnected(): # send message only if the user is connected
			date = datetime.datetime.now().isoformat()

			self.client.send(xmpp.protocol.Message(to, body, type, subject, {}, self.jid, [], date))

			date = date.replace(':', '-')
			
			# save the message
			self.save_sent_message(datetime.datetime.strptime(date.split('.')[0], "%Y-%m-%dT%H-%M-%S").strftime("%d-%m-%Y %H.%M.%S"), to, subject, body)

			if self.debug: print _("I send:\nDate: ") + date + _("\nTo: ") + to + _("\nType: ") + type + _("\nSubject: ") + subject + _("\nBody:\n") + body

	# return the avatar type
	def get_avatar_type(self, file):
		ext = os.path.splitext(file)
		
		if ext[1] == ".jpg" or ext[1] == ".jpeg": return "image/jpeg"
		elif ext[1] == ".png": return "image/png"
		else: return ""
	
	# connect to jabber server
	def jabber_connect(self):
		if self.is_connecting:
			return False

		self.is_connecting = True

		try:
			# check for required fields
			if self.myname == "" or self.username == "" or self.domain == "" or self.password == "" or self.server == "" or str(self.port) == "":
				self.message_box(_("You must fill in the required fields to make the connection (name, username, password, server, port)"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winMain)
				self.is_connecting = False
				return False
			
			self.jid = xmpp.protocol.JID(self.username + "@" + self.domain + "/pygeon")
			self.client = xmpp.Client(self.jid.getDomain(), debug=[])

			# if it doesn't connect to the server
			if not self.client.connect(server=(self.server, self.port)):
				if not self.reconnecting:
					self.message_box(_("Connection with the server ") + self.server + _(" failed"), _("Warning"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)
					self.statusbar.push(0, _("Connection with ") + self.server + _(" failed"))
					if self.debug: print _("Connection with ") + self.server + _(" failed")
				else:
					self.message_box(_("Connection with the server ") + self.server + _(" broken for network problems"), _("Warning"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)
					self.statusbar.push(0, _("Connection with ") + self.server + _(" broken"))
					if self.debug: print _("Connection with ") + self.server + _(" broken")
					self.reconnecting = False

				self.is_connecting = False
				return False

			# if it doesn't authenticate on the server
			if not self.client.auth(self.jid.getNode(), self.password, self.jid.getResource()):
				if not self.reconnecting:
					self.message_box(_("Authentication on ") + self.server + _(" failed"), _("Warning"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)
					self.statusbar.push(0, _("Authentication on ") + self.server + _(" failed"))
					if self.debug: print _("Authentication on ") + self.server + _(" failed")
				else:
					self.message_box(_("Connection with the server ") + self.server + _(" broken for network problems"), _("Warning"), gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, self.winMain)
					self.statusbar.push(0, _("Connection with ") + self.server + _(" broken"))
					if self.debug: print _("Connection with ") + self.server + _(" broken")
					self.reconnecting = False

				self.is_connecting = False
				return False

			self.client.RegisterHandler('iq', self.receive_iq) # iq handler
			self.client.RegisterHandler('message', self.receive_message) # message handler
			self.client.RegisterHandler('presence', self.receive_presence) # presence handler

			# send the avatar to the server if necessary
			if not self.avatar_uploaded:
				# move the avatar in the temporary var to wait the ok from the server
				self.avatar_temp = self.avatar
				self.avatar = ""

				if self.avatar_temp == "" or not os.path.isfile(self.avatar_temp):
					self.avatar_temp = os.path.normpath(self.icon_theme_application_path + "no_avatar.png")

				if self.ddebug: print _("I am about to send to the server: ") + self.avatar_temp

				f = open(self.avatar_temp, 'rb')
				stringa = f.read()
				codifica = base64.encodestring(stringa)
				f.close()

				iq = xmpp.protocol.Iq(typ = 'set')
				tag = iq.setTag("vcard-temp vCard")
				child = tag.addChild("PHOTO")
				child.addChild("TYPE").setData(self.get_avatar_type(self.avatar_temp))

				if self.avatar_temp != os.path.normpath(self.icon_theme_application_path + "no_avatar.png"):
					child.addChild("BINVAL").setData(codifica)
				else:
					child.addChild("BINVAL")
				
				self.avatar_uploaded = True

				# save to db
				cursor = self.conn.cursor()
				cursor.execute("update preferences set avatar_uploaded = ? where id = 1", (int(self.avatar_uploaded),))
				self.conn.commit()
				cursor.close()

				# send the message with avatar to the server
				self.client.send(iq)

				if self.debug: print _("I sent the avatar to the server")

			# send the presence message manually to send the avatar hash if it exists
			rosters = self.client.getRoster()
			self.change_status(self.cboStatus.get_active())

			# listen to the messages
			gobject.timeout_add(50, self.jabber_listen().next)

			# check unread messages
			gobject.timeout_add(500, self.check_unread_messages)

			# check unread messages to load the counters for new message (start after 1 second and act only one time)
			self.message_counter = False # reset the variable
			gobject.timeout_add(1000, self.load_message_counter)

			# check if network is ok
			self.reconnecting = False
			self.previous_check = None
			gobject.timeout_add(30000, self.check_network)

			# after 10 seconds sets just_connected to False
			self.just_connected = True
			gobject.timeout_add(10000, self.set_just_connected)

			self.statusbar.push(0, _("Connected to the server ") + self.server)
			if self.debug: print _("Connected to the jabber server")

			self.is_connecting = False
			return True
		except Exception:
			if self.debug: print _("Connection to the server failed")

			if self.reconnecting: # if it has tried to reconnect
				self.message_box(_("Connected with the server ") + self.server + _("  broken for network problems"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winMain)
				self.statusbar.push(0, _("Connection with ") + self.server + _(" broken"))
				self.reconnecting = False
			else: # normal connection
				self.message_box(_("Connection with the server ") + self.server + _(" failed"), _("Warning"), gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, self.winMain)
				self.statusbar.push(0, _("Connected with ") + self.server + _(" failed"))

			self.is_connecting = False
			return False

	# disconnect from jabber server
	def jabber_disconnect(self):
		self.client.disconnect()
		self.lstUtenti.get_model().clear() # empty contact list

		self.statusbar.push(0, _("Disconnected from the server ") + self.server)
		if self.debug: print _("Disconnected from the jabber server")

	# get the contact name from username (from the contact list)
	def get_name_from_username(self, username):
		iter = self.lstUtenti.get_model().get_iter_first()

		found = username

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == username:
				found = self.lstUtenti.get_model().get_value(iter, 1).split('\n')[0]
				break

			iter = self.lstUtenti.get_model().iter_next(iter)

		return found

	# remove the roster from the contact list
	def remove_roster_from_list(self, jid):
		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if self.lstUtenti.get_model().get_value(iter, 4) == jid:
				self.lstUtenti.get_model().remove(iter)

				if self.debug: print _("I deleted the user from the list: ") + jid

				break

			iter = self.lstUtenti.get_model().iter_next(iter)			

	# sort contact list (alphabetically => 1, by status => 7)
	def sort_roster_list(self):
		if self.sort_method == 7:
			self.lstUtenti.get_model().set_sort_column_id(7, gtk.SORT_ASCENDING)
		else:
			self.lstUtenti.get_model().set_sort_column_id(1, gtk.SORT_ASCENDING)

	# change the roster status
	# if only_icon is True change only the status icon
	def change_roster_status(self, roster, status, info=None, only_icon=False):
		# create the icon 
		if status == 0: pix = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "available.png"))
		elif status == 1: pix = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "away.png"))
		elif status == 2: pix = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "dnd.png"))
		elif status == 3: pix = gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png"))

		iter = self.lstUtenti.get_model().get_iter_first()

		while not iter is None:
			if str(roster).startswith(self.lstUtenti.get_model().get_value(iter, 4)): 
				
				# if there is an additional information add it under the roster name
				if not only_icon:
					if not info is None: self.lstUtenti.get_model().set_value(iter, 1, str(self.lstUtenti.get_model().get_value(iter, 1)).split('\n')[0] + "\n<span foreground=\"#999999\"><small><i>" + str(info) + "</i></small></span>")
					else: # put only the name
						nome = self.client.getRoster().getName(str(roster))

						if self.lstUtenti.get_model().get_value(iter, 1) != nome:
							self.lstUtenti.get_model().set_value(iter, 1, nome)
							if self.debug: print _("I leave only the username of: ") + nome
				
				# change status icon and status
				# the icon change only if the user has no new messages into the counter
				if self.lstUtenti.get_model().get_value(iter, 5) == 0: self.lstUtenti.get_model().set_value(iter, 0, pix)
				if not only_icon:
					# legge lo status vecchio
					# read the old status
					old_status = self.lstUtenti.get_model().get_value(iter, 3)

					# set the new status
					self.lstUtenti.get_model().set_value(iter, 3, status)

					# cambia anche il campo status+utente per l'ordinamento
					# change also the field status+user for sorting
					self.lstUtenti.get_model().set_value(iter, 7, str(status) + self.lstUtenti.get_model().get_value(iter, 1))

					if status == 3: # if it's disconnection add tooltip
						self.lstUtenti.get_model().set_value(iter, 6, self.read_last_seen(str(roster).partition('/')[0]))
					elif status != 3 and old_status == 3: # from disconnected to connected remove tooltip
						self.lstUtenti.get_model().set_value(iter, 6, None)

				break

			iter = self.lstUtenti.get_model().iter_next(iter)

		if self.debug: print _("I change the status of ") + str(roster) + _(" into ") + str(status)

	# status bar counter. counts the number of connected users and total users
	def count_rosters(self):
		iter = self.lstUtenti.get_model().get_iter_first()

		tot_users = 0
		con_users = 0

		while not iter is None:
			status = self.lstUtenti.get_model().get_value(iter, 3)
			if status != 3:
				con_users = con_users + 1

			tot_users = tot_users + 1

			iter = self.lstUtenti.get_model().iter_next(iter)

		self.statusbar.push(0, _("Connected to the server ") + self.server + " (" + str(con_users) + "/" + str(tot_users) + ")")

	# doppio click su un utente nella lista contatti
	# double click on a user in the contact list
	def lstUtenti_doubleclick(self, widget, path, column):
		row = self.lstUtenti.get_selection().get_selected()

		if not row is None:
			utente = row[0].get_value(row[1], 4)

			# open the send message window if there aren't new messages/chat, otherwise the unread messages window
			if row[0].get_value(row[1], 5) == 0 and row[0].get_value(row[1], 8) == 0: self.send_message_classes.append(send_message(self, utente))
			else:
				# open read message window if there are new messages
				if row[0].get_value(row[1], 5) > 0:
					classe = self.get_read_message(2, utente)

					if classe is None:
						self.read_message_classes.append(read_message(self, 2, utente))
					else:
						classe.winRead.present()
				
				# open all new chats
				if row[0].get_value(row[1], 8) > 0:
					for classe in self.chats:
						if classe.to == utente and not classe.winChat.get_property("visible"):
							classe.winChat.show()
							self.remove_chat_from_contact(utente)

	# click on send message
	def mnuInviaMessaggio_activate(self, widget, utente):
		self.send_message_classes.append(send_message(self, utente))

	# click on send file
	def mnuInviaFile_activate(self, widget, utente):
		f = self.file_chooser(_("Choose file"), self.winMain, gtk.FILE_CHOOSER_ACTION_OPEN)

		# check the file is valid
		if f != "" and os.path.isfile(f):
			# generate a random id for file transfer
			id = self.generate_chat_thread()

			# get file size
			size = os.path.getsize(f)
	
			# add a file transfer class to the list
			self.file_transfers.append(file_transfer(self, fid=id, destinatario=utente, filename=f))
			if self.debug: print _("I add a file transfer class. Now the list size is: ") + str(len(self.file_transfers))

			# send a file transfer request message
			self.send_file_transfer_request(to=utente, filename=os.path.basename(f), fid=id, size=size)

	# open chat windows of 'utente'
	def mnuConversazioniNonLette_activate(self, widget, utente, counter):
		# if there aren't new chat show a warning message
		if counter == 0: self.message_box(_("No new conversations"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			for classe in self.chats:
				if classe.to == utente and not classe.winChat.get_property("visible"):
					classe.winChat.show()
					self.remove_chat_from_contact(utente)

	# open the read message window for 'utente'
	def mnuMessaggiNonLetti_activate(self, widget, utente, counter):
		# if there aren't new chat show a warning message
		if counter == 0: self.message_box(_("No new messages"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_read_message(2, utente)

			if classe is None:
				self.read_message_classes.append(read_message(self, 2, utente))
			else:
				classe.winRead.present()

	# open the window to read the messages sent to 'utente'
	def mnuMessaggiInviati_activate(self, widget, utente):
		if self.count_messages(0, utente) == 0: self.message_box(_("No message has been sent"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_read_message(0, utente)

			if classe is None:
				self.read_message_classes.append(read_message(self, 0, utente))
			else:
				classe.winRead.present()

	# open the window to read the messages received from 'utente'
	def mnuMessaggiRicevuti_activate(self, widget, utente):
		if self.count_messages(1, utente) == 0: self.message_box(_("No message has been received"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_read_message(1, utente)

			if classe is None:
				self.read_message_classes.append(read_message(self, 1, utente))
			else:
				classe.winRead.present()

	# mark messages of 'utente' as read
	def mnuMarcaMessaggi_activate(self, widget, utente):
		cursor = self.conn.cursor()
		cursor.execute("update messages set type = 1 where with like '" + self.tdb(utente) + "%'")
		self.conn.commit()
		cursor.close()

		# reset the counter
		self.remove_message_from_contact(utente, True)

	# generate a thread for chat
	def generate_chat_thread(self):
		import random
		return ''.join([random.choice(string.ascii_letters) for x in xrange(0,32)])

	# get the position of the chat in the chats list through 'utente'
	# if it doesn't find the chat return -1
	def get_chat_from_user(self, utente):
		position = -1

		for i in range(len(self.chats)):
			if self.chats[i].to == utente:
				position = i
				break

		return position
	
	# get the position of the chat in the chats list through 'thread'
	# if it doesn't find the chat return -1
	def get_chat_from_thread(self, thread):
		position = -1

		for i in range(len(self.chats)):
			if self.chats[i].thread == thread:
				position = i
				break

		return position

	# get the position of the file transfer class list through 'id'
	def get_file_transfer(self, id):
		position = -1

		for i in range(len(self.file_transfers)):
			if self.file_transfers[i].fid == id:
				position = i
				break

		return position

	# remove the class with 'thread' from the chats list
	def remove_chat(self, thread):
		for i in range(len(self.chats)):
			if self.chats[i].thread == thread:
				del self.chats[i]
				
				if self.debug: print _("I removed the chat with thread ") + str(thread) + _(" from the list.\nNow the lenght of the list is: ") + str(len(self.chats))
				break

	# open a new chat window (if a window for the user is already opened present it)
	def mnuChat_activate(self, widget, utente):
		position = self.get_chat_from_user(utente)

		if position == -1: # create the chat
			if self.debug: print _("The chat window for ") + utente + _(" does not exist yet. I create it...")
			self.chats.append(chat(self, utente, self.generate_chat_thread()))
		else: # present the window
			if self.debug: print _("The chat window for ") + utente + _(" exists and it is in position ") + str(position) + _(". I present the window...")
			
			if not self.chats[position].winChat.get_property("visible"):
				self.remove_chat_from_contact(utente)

			self.chats[position].winChat.present()

	# open chat chronology for 'utente'
	def mnuCronologiaChat_activate(self, widget, utente):
		if self.count_chats(utente) == 0: self.message_box(_("No conversations"), _("Warning"), gtk.MESSAGE_INFO, gtk.BUTTONS_OK, self.winMain)
		else:
			classe = self.get_crono(utente)

			if classe is None:			
				self.crono_classes.append(crono(self, utente))
			else:
				classe.winCrono.present()

	# open the information window
	def mnuInformazioni_activate(self, widget, utente):
		user_info(self, utente)

	# right click on a contact in the contact list
	def lstUtenti_rightclick(self, widget, event):
		if event.button == 3: # right click
			row = self.lstUtenti.get_selection().get_selected()

			if not row[1] is None:
				utente = row[0].get_value(row[1], 4)
				counter = row[0].get_value(row[1], 5)
				counter_chats = row[0].get_value(row[1], 8)
				status = row[0].get_value(row[1], 3)

				# create the popup menu
				self.lstMenu = gtk.Menu()

				# send message
				mnuInviaMessaggio = gtk.ImageMenuItem(_("Send message"))
				imgInviaMessaggio = gtk.Image()
				imgInviaMessaggio.set_from_file(os.path.normpath(self.icon_theme_application_path + "message.png"))
				mnuInviaMessaggio.set_image(imgInviaMessaggio)
				mnuInviaMessaggio.connect("activate", self.mnuInviaMessaggio_activate, utente)
				mnuInviaMessaggio.set_property("always-show-image", True)
				self.lstMenu.append(mnuInviaMessaggio)

				# send file
				mnuInviaFile = gtk.ImageMenuItem(_("Send file"))
				imgInviaFile = gtk.Image()
				imgInviaFile.set_from_file(os.path.normpath(self.icon_theme_application_path + "file_transfer.png"))
				mnuInviaFile.set_image(imgInviaFile)
				mnuInviaFile.connect("activate", self.mnuInviaFile_activate, utente)
				mnuInviaFile.set_property("always-show-image", True)
				self.lstMenu.append(mnuInviaFile)
				if status == 3: mnuInviaFile.set_sensitive(False) # disable send file is the user is offline 

				# chat	
				mnuChat = gtk.ImageMenuItem(_("Open chat"))
				imgChat = gtk.Image()
				imgChat.set_from_file(os.path.normpath(self.icon_theme_application_path + "chat.png"))
				mnuChat.set_image(imgChat)
				mnuChat.connect("activate", self.mnuChat_activate, utente)
				mnuChat.set_property("always-show-image", True)
				self.lstMenu.append(mnuChat)

				# separator
				sepChat = gtk.SeparatorMenuItem()
				self.lstMenu.append(sepChat)

				# unread conversations
				mnuConversazioniNonLette = gtk.ImageMenuItem(_("Unread conversations"))
				imgConversazioniNonLette = gtk.Image()
				imgConversazioniNonLette.set_from_file(os.path.normpath(self.icon_theme_application_path + "new_conversations.png"))
				mnuConversazioniNonLette.set_image(imgConversazioniNonLette)
				mnuConversazioniNonLette.connect("activate", self.mnuConversazioniNonLette_activate, utente, counter_chats)
				mnuConversazioniNonLette.set_property("always-show-image", True)
				self.lstMenu.append(mnuConversazioniNonLette)

				# unread messages
				mnuMessaggiNonLetti = gtk.ImageMenuItem(_("Unread messages"))
				imgMessaggiNonLetti = gtk.Image()
				imgMessaggiNonLetti.set_from_file(os.path.normpath(self.icon_theme_application_path + "unread_messages.png"))
				mnuMessaggiNonLetti.set_image(imgMessaggiNonLetti)
				mnuMessaggiNonLetti.connect("activate", self.mnuMessaggiNonLetti_activate, utente, counter)
				mnuMessaggiNonLetti.set_property("always-show-image", True)
				self.lstMenu.append(mnuMessaggiNonLetti)
				
				# sent messages
				mnuMessaggiInviati = gtk.ImageMenuItem(_("Sent messages"))
				imgMessaggiInviati = gtk.Image()
				imgMessaggiInviati.set_from_file(os.path.normpath(self.icon_theme_application_path + "sent_messages.png"))
				mnuMessaggiInviati.set_image(imgMessaggiInviati)
				mnuMessaggiInviati.connect("activate", self.mnuMessaggiInviati_activate, utente)
				mnuMessaggiInviati.set_property("always-show-image", True)
				self.lstMenu.append(mnuMessaggiInviati)
				
				# received messages
				mnuMessaggiRicevuti = gtk.ImageMenuItem(_("Received messages"))
				imgMessaggiRicevuti = gtk.Image()
				imgMessaggiRicevuti.set_from_file(os.path.normpath(self.icon_theme_application_path + "received_messages.png"))
				mnuMessaggiRicevuti.set_image(imgMessaggiRicevuti)
				mnuMessaggiRicevuti.connect("activate", self.mnuMessaggiRicevuti_activate, utente)
				mnuMessaggiRicevuti.set_property("always-show-image", True)
				self.lstMenu.append(mnuMessaggiRicevuti)
				
				# mark messages as read
				mnuMarcaMessaggi = gtk.ImageMenuItem(_("Mark messages as read"))
				imgMarcaMessaggi = gtk.Image()
				imgMarcaMessaggi.set_from_file(os.path.normpath(self.icon_theme_application_path + "mark_messages_as_read.png"))
				mnuMarcaMessaggi.set_image(imgMarcaMessaggi)
				if counter == 0: mnuMarcaMessaggi.set_sensitive(False) # disable menu if there aren't new messages
				mnuMarcaMessaggi.connect("activate", self.mnuMarcaMessaggi_activate, utente)
				mnuMarcaMessaggi.set_property("always-show-image", True)
				self.lstMenu.append(mnuMarcaMessaggi)
				
				# chat chronology
				mnuCronologiaChat = gtk.ImageMenuItem(_("Chat chronology"))
				imgCronologiaChat = gtk.Image()
				imgCronologiaChat.set_from_file(os.path.normpath(self.icon_theme_application_path + "chat_history.png"))
				mnuCronologiaChat.set_image(imgCronologiaChat)
				mnuCronologiaChat.connect("activate", self.mnuCronologiaChat_activate, utente)
				mnuCronologiaChat.set_property("always-show-image", True)
				self.lstMenu.append(mnuCronologiaChat)
				
				# separator
				sepMarcaMessaggi = gtk.SeparatorMenuItem()
				self.lstMenu.append(sepMarcaMessaggi)				
				
				# informations
				mnuInformazioni = gtk.ImageMenuItem(_("Information"))
				imgInformazioni = gtk.Image()
				imgInformazioni.set_from_file(os.path.normpath(self.icon_theme_application_path + "user_info.png"))
				mnuInformazioni.set_image(imgInformazioni)
				mnuInformazioni.connect("activate", self.mnuInformazioni_activate, utente)
				mnuInformazioni.set_property("always-show-image", True)
				self.lstMenu.append(mnuInformazioni)

				self.lstMenu.show_all()
				self.lstMenu.popup(None, None, None, event.button, event.time)

	# when appear the window it will be positioned at south-east
	def show_window(self, widget):
		width, height = self.winMain.get_size()
		self.winMain.move(gtk.gdk.screen_width() - width, gtk.gdk.screen_height() - height - 50)

	# constructor
	def __init__(self):
		if os.name == 'nt':
			# hide windows console when the program start
			import win32gui as win32

			hWnd = None
			while hWnd is None:
				hWnd = win32.FindWindow(None, "pygeon")

				try:
					win32.ShowWindow(hWnd, 0); # 0 = SW_HIDE
				except Exception:
					pass

		# check parameters
		for arg in sys.argv:
			if arg == "-d":
				self.debug = True
			elif arg == "-dd":
				self.debug = True
				self.ddebug = True
			elif arg == "-m":
				self.mute = True
		
		# set paths depending on operating system
		if os.name == "posix":
			self.pref_path = os.path.expanduser('~') + "/.pygeon/"
			self.current_path = os.getcwd() + "/"
			signal.signal(signal.SIGHUP, self.sighup_handler) # linux signal
		else:
			self.pref_path = os.environ['APPDATA'] + "\\pygeon\\"
			self.current_path = os.getcwd() + "\\"
			signal.signal(signal.SIGBREAK, self.sigbreak_handler) # windows signal
		
		# check if pygration is needed	
		if os.path.isdir(self.pref_path) and not os.path.isfile(self.pref_path + "pygeon.db"):
			self.pygration = True
		else:
			self.pygration = False

			# create preferences if necessary
			if not os.path.isfile(self.pref_path + "pygeon.db"):
				self.create_preferences()
			else: # otherwise connects to db
				self.conn = self.connect_to_db()

			# add column to db table if preferences are older than new version
			try:
				cursor = self.conn.cursor()
				cursor.execute("select sort_method from preferences where id = 1")
				cursor.close()
			except:			
				cursor = self.conn.cursor()
				cursor.execute("alter table preferences add column sort_method integer")
				self.conn.commit()

				cursor.execute("update preferences set sort_method = 1 where id = 1")
				self.conn.commit()

				cursor.close()
			
			try:
				cursor = self.conn.cursor()
				cursor.execute("select notify_normal from preferences where id = 1")
				cursor.close()
			except:			
				cursor = self.conn.cursor()
				cursor.execute("alter table preferences add column notify_normal integer")
				cursor.execute("alter table preferences add column notify_chat integer")
				cursor.execute("alter table preferences add column notify_connect integer")
				cursor.execute("alter table preferences add column notify_disconnect integer")
				cursor.execute("alter table preferences add column notify_status integer")
				cursor.execute("alter table preferences add column notify_add integer")
				cursor.execute("alter table preferences add column notify_remove integer")
				self.conn.commit()

				cursor.execute("update preferences set notify_normal = 1 where id = 1")
				cursor.execute("update preferences set notify_chat = 1 where id = 1")
				cursor.execute("update preferences set notify_connect = 0 where id = 1")
				cursor.execute("update preferences set notify_disconnect = 0 where id = 1")
				cursor.execute("update preferences set notify_status = 0 where id = 1")
				cursor.execute("update preferences set notify_add = 1 where id = 1")
				cursor.execute("update preferences set notify_remove = 1 where id = 1")
				self.conn.commit()

				cursor.close()
			
			try:
				cursor = self.conn.cursor()
				cursor.execute("select notify_transfer from preferences where id = 1")
				cursor.close()
			except:			
				cursor = self.conn.cursor()
				cursor.execute("alter table preferences add column notify_transfer integer")
				self.conn.commit()

				cursor.execute("update preferences set notify_transfer = 1 where id = 1")
				self.conn.commit()

				cursor.close()

			try:
				cursor = self.conn.cursor()
				cursor.execute("select theme from preferences where id = 1")
				cursor.close()
			except:
				cursor = self.conn.cursor()
				cursor.execute("alter table preferences add column theme varchar")
				self.conn.commit()

				cursor.execute("update preferences set theme = 'default' where id = 1")
				self.conn.commit()

				cursor.close()

			# load preferences
			self.load_preferences()
			
		self.icon_application_path = os.path.normpath(self.current_path + "../data/images/app") + "/"
		self.icon_theme_application_path = os.path.normpath(self.current_path + "../data/images/themes/" + self.theme + "/app") + "/"
		self.icon_theme_status_path = os.path.normpath(self.current_path + "../data/images/themes/" + self.theme + "/status") + "/"
		self.icon_theme_emoticons_path = os.path.normpath(self.current_path + "../data/images/themes/" + self.theme + "/emotes") + "/"

		if self.debug: print _("The system is ") + os.name
		if self.debug: print _("User pygeon folder: ") + self.pref_path
		if self.debug: print _("Current folder: ") + self.current_path
		
		self.gladefile = "../data/glade/pygeon.glade"
		self.winContainer = gtk.glade.XML(self.gladefile, "winMain", "pygeon")
	
		# glade objects
		self.winMain = self.winContainer.get_widget("winMain")
		self.mnuEsci = self.winContainer.get_widget("mnuEsci")
		self.mnuBroadcast = self.winContainer.get_widget("mnuBroadcast")
		self.mnuMulti = self.winContainer.get_widget("mnuMulti")
		self.mnuInviati = self.winContainer.get_widget("mnuInviati")
		self.mnuRicevuti = self.winContainer.get_widget("mnuRicevuti")
		self.mnuFiletransf = self.winContainer.get_widget("mnuFiletransf")
		self.mnuFile = self.winContainer.get_widget("mnuFile")
		self.mnuEdit = self.winContainer.get_widget("mnuEdit")
		self.mnuTools = self.winContainer.get_widget("mnuTools")
		self.mnuHelp = self.winContainer.get_widget("mnuHelp")
		self.mnuBug = self.winContainer.get_widget("mnuBug")
		self.mnuInfo = self.winContainer.get_widget("mnuInfo")
		self.mnuSort = self.winContainer.get_widget("mnuSort")
		self.mnuAlphabetic = self.winContainer.get_widget("mnuAlphabetic")
		self.mnuSortstatus = self.winContainer.get_widget("mnuSortstatus")
		self.mnuPref = self.winContainer.get_widget("mnuPref")
		self.mnuCrono = self.winContainer.get_widget("mnuCron")
		self.mnuAiuto = self.winContainer.get_widget("mnuAiuto")
		self.imgSort = self.winContainer.get_widget("imgSort")
		self.imgBug = self.winContainer.get_widget("imgBug")
		self.imgBroadcast = self.winContainer.get_widget("imgBroadcast")
		self.imgMulti = self.winContainer.get_widget("imgMulti")
		self.imgInviati = self.winContainer.get_widget("imgInviati")
		self.imgRicevuti = self.winContainer.get_widget("imgRicevuti")
		self.imgFiletransf = self.winContainer.get_widget("imgFiletransf")
		self.imgCron = self.winContainer.get_widget("imgCron")
		self.imgQuit = self.winContainer.get_widget("imgQuit")
		self.imgPreferences = self.winContainer.get_widget("imgPreferences")
		self.imgHelp = self.winContainer.get_widget("imgHelp")
		self.imgAbout = self.winContainer.get_widget("imgAbout")
		self.cboStatus = self.winContainer.get_widget("cboStatus")
		self.lstUtenti = self.winContainer.get_widget("lstUtenti")
		self.statusbar = self.winContainer.get_widget("statusbar")

		self.mnuFile.get_children()[0].set_label(_('_File'))
		self.mnuEdit.get_children()[0].set_label(_('_Edit'))
		self.mnuTools.get_children()[0].set_label(_('_Tools'))
		self.mnuHelp.get_children()[0].set_label(_('_Help'))
		self.mnuEsci.get_children()[0].set_label(_('_Quit'))
		self.mnuSort.get_children()[0].set_label(_('Sort contacts'))
		self.mnuAlphabetic.get_children()[0].set_label(_('Alphabetically'))
		self.mnuSortstatus.get_children()[0].set_label(_('By status'))
		self.mnuPref.get_children()[0].set_label(_('_Preferences'))
		self.mnuBroadcast.get_children()[0].set_label(_('Send broadcast'))
		self.mnuMulti.get_children()[0].set_label(_('Multiple message'))
		self.mnuInviati.get_children()[0].set_label(_('Sent messages'))
		self.mnuRicevuti.get_children()[0].set_label(_('Received messages'))
		self.mnuFiletransf.get_children()[0].set_label(_('File transfer'))
		self.mnuCrono.get_children()[0].set_label(_('Chronology manager'))
		self.mnuAiuto.get_children()[0].set_label(_('Help'))
		self.mnuInfo.get_children()[0].set_label(_('Information'))
		self.mnuBug.get_children()[0].set_label(_('Report a bug'))

		# signals
		self.winMain.connect("delete_event", self.delete_event)
		self.winMain.connect("destroy", self.destroy)
		self.winMain.connect("show", self.show_window)
		self.mnuAiuto.connect("activate", self.mnuAiuto_activate)
		self.mnuBug.connect("activate", self.mnuBug_activate)
		self.mnuEsci.connect("activate", self.mnuEsci_activate)
		self.mnuBroadcast.connect("activate", self.mnuBroadcast_activate)
		self.mnuMulti.connect("activate", self.mnuMulti_activate)
		self.mnuInviati.connect("activate", self.mnuInviati_activate)
		self.mnuRicevuti.connect("activate", self.mnuRicevuti_activate)
		self.mnuFiletransf.connect("activate", self.mnuFiletransf_activate)
		self.mnuInfo.connect("activate", self.mnuInfo_activate)
		self.mnuPref.connect("activate", self.mnuPref_activate)
		self.mnuAlphabetic.connect("activate", self.mnuAlphabetic_activate)
		self.mnuSortstatus.connect("activate", self.mnuSortstatus_activate)
		self.mnuCrono.connect("activate", self.mnuCrono_activate)
		self.cboStatus.connect("changed", self.cboStatus_changed)
		self.lstUtenti.connect("row_activated", self.lstUtenti_doubleclick)
		self.lstUtenti.connect("button_release_event", self.lstUtenti_rightclick)

		# hide window (is in the tray)
		self.winMain.hide()
		
		self.imgSort.set_from_file(os.path.normpath(self.icon_theme_application_path + "sort_asc.png"))
		self.imgBug.set_from_file(os.path.normpath(self.icon_theme_application_path + "bug.png"))
		self.imgBroadcast.set_from_file(os.path.normpath(self.icon_theme_application_path + "broadcast.png"))
		self.imgMulti.set_from_file(os.path.normpath(self.icon_theme_application_path + "multi_message.png"))
		self.imgInviati.set_from_file(os.path.normpath(self.icon_theme_application_path + "sent_messages.png"))
		self.imgRicevuti.set_from_file(os.path.normpath(self.icon_theme_application_path + "received_messages.png"))
		self.imgFiletransf.set_from_file(os.path.normpath(self.icon_theme_application_path + "file_transfer.png"))
		self.imgCron.set_from_file(os.path.normpath(self.icon_theme_application_path + "chat_history.png"))
		self.imgQuit.set_from_file(os.path.normpath(self.icon_theme_application_path + "quit.png"))
		self.imgPreferences.set_from_file(os.path.normpath(self.icon_theme_application_path + "preferences.png"))
		self.imgHelp.set_from_file(os.path.normpath(self.icon_theme_application_path + "help.png"))
		self.imgAbout.set_from_file(os.path.normpath(self.icon_theme_application_path + "about.png"))
		self.winMain.set_icon_from_file(os.path.normpath(self.icon_application_path + "pygeon-small.png"))
		self.winMain.set_title(_("Pygeon"))
		self.winMain.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

		# build combo for of status
		store = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING) # image and text of the status
		self.cboStatus.set_model(store)
		cell_pixbuf = gtk.CellRendererPixbuf()
		cell_text = gtk.CellRendererText()
		self.cboStatus.pack_start(cell_pixbuf, False)
		self.cboStatus.pack_start(cell_text, False)
		self.cboStatus.add_attribute(cell_pixbuf, "pixbuf", 0)
		self.cboStatus.add_attribute(cell_text, "text", 1)

		store.append((gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "available.png")), _("Available")))
		store.append((gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "away.png")), _("Away")))
		store.append((gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "dnd.png")), _("Not available")))
		store.append((gtk.gdk.pixbuf_new_from_file(os.path.normpath(self.icon_theme_status_path + "offline.png")), _("Offline")))

		# build the contact list (status image, name, avatar, status, username, number of new mex, tooltip, status+name, number of new chats)
		store = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING, gtk.gdk.Pixbuf, gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT)
		self.lstUtenti.set_model(store)

		# name column
		render_nome = gtk.CellRendererText()
		render_nome.set_property("wrap_mode", pango.WRAP_WORD) # text wrap
		render_nome.set_property("wrap_width", 190) # cell dimension
		colonna_nome = gtk.TreeViewColumn("", render_nome, markup=1)
		colonna_nome.set_max_width(190) # column dimension

		render_avatar = gtk.CellRendererPixbuf()
		render_avatar.set_property("xalign", 1.0)
		colonna_avatar = gtk.TreeViewColumn("", render_avatar, pixbuf=2)
		colonna_avatar.set_alignment(0.0)

		self.lstUtenti.append_column(gtk.TreeViewColumn("", gtk.CellRendererPixbuf(), pixbuf=0))
		self.lstUtenti.append_column(colonna_nome)
		self.lstUtenti.append_column(colonna_avatar)

		colonna_status = gtk.TreeViewColumn("", gtk.CellRendererText(), text=3)
		colonna_username = gtk.TreeViewColumn("", gtk.CellRendererText(), text=4)
		colonna_messaggi = gtk.TreeViewColumn("", gtk.CellRendererText(), text=5)
		colonna_tooltip = gtk.TreeViewColumn("", gtk.CellRendererText(), text=6)
		colonna_status_and_username = gtk.TreeViewColumn("", gtk.CellRendererText(), text=7) # need for sorting
		colonna_chat = gtk.TreeViewColumn("", gtk.CellRendererText(), text=8)
		colonna_status.set_visible(False)
		colonna_username.set_visible(False)
		colonna_messaggi.set_visible(False)
		colonna_tooltip.set_visible(False)
		colonna_status_and_username.set_visible(False)
		colonna_chat.set_visible(False)

		self.lstUtenti.append_column(colonna_status)
		self.lstUtenti.append_column(colonna_username)
		self.lstUtenti.append_column(colonna_messaggi)
		self.lstUtenti.append_column(colonna_tooltip)
		self.lstUtenti.append_column(colonna_status_and_username)
		self.lstUtenti.append_column(colonna_chat)

		self.lstUtenti.set_tooltip_column(6)
		
		# load preferences
		self.load_preferences()
		
		# call tray icon class
		self.tray = tray(self)
		
		# autoconnect if the option is True
		if self.autoconnect: self.cboStatus.set_active(0)
		else: self.cboStatus.set_active(3)

		# sort method
		if self.sort_method == 7:
			self.mnuSortstatus.set_active(True)
		else:
			self.mnuAlphabetic.set_active(True)
	
	# windows closing
	def sigbreak_handler(self, signum, frame):
		self.close_program()
		sys.exit() # exit program to avoid windows blocking

	# linux closing
	# GNOME and XFCE: useful only when the program is started into a console
	# KDE: always (graphic too)
	def sighup_handler(self, signum, frame):
		self.close_program()

	# end of gnome session (when the program in GNOME/XFCE is started graphically and not into a console)
	def end_gnome_session(self, *args):
		self.close_program()

	# function that is called by the handlers when the program is closed
	def close_program(self):
		# save preferences and last seen
		self.save_last_seen()

		# close db connection
		self.disconnect_from_db(self.conn)
	
	def main(self):
		# add program to gnome session to catch the logout signal if possible
		try:
			import gnome.ui
			gnome.program_init('pygeon.py', self.program_version)
			cl = gnome.ui.master_client()
			cl.connect("die", self.end_gnome_session)
		except:
			pass
			
		# close the program if pygration is needed
		if self.pygration:
			# show a message before closing
			self.message_box("Pygeon data must be converted to new format using pygration tool. The program will be closed", "Warning", gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, None)
		else:
			gtk.main()

if __name__ == "__main__":
	pygeon = pygeon()
	pygeon.main()
